# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.automl.v1",
    manifest={
        "InputConfig",
        "BatchPredictInputConfig",
        "DocumentInputConfig",
        "OutputConfig",
        "BatchPredictOutputConfig",
        "ModelExportOutputConfig",
        "GcsSource",
        "GcsDestination",
    },
)


class InputConfig(proto.Message):
    r"""Input configuration for
    [AutoMl.ImportData][google.cloud.automl.v1.AutoMl.ImportData]
    action.

    The format of input depends on dataset_metadata the Dataset into
    which the import is happening has. As input source the
    [gcs_source][google.cloud.automl.v1.InputConfig.gcs_source] is
    expected, unless specified otherwise. Additionally any input .CSV
    file by itself must be 100MB or smaller, unless specified otherwise.
    If an "example" file (that is, image, video etc.) with identical
    content (even if it had different ``GCS_FILE_PATH``) is mentioned
    multiple times, then its label, bounding boxes etc. are appended.
    The same file should be always provided with the same ``ML_USE`` and
    ``GCS_FILE_PATH``, if it is not, then these values are
    nondeterministically selected from the given ones.

    The formats are represented in EBNF with commas being literal and
    with non-terminal symbols defined near the end of this comment. The
    formats are:

    .. raw:: html

        <h4>AutoML Vision</h4>

    .. raw:: html

        <div class="ds-selector-tabs"><section><h5>Classification</h5>

    See `Preparing your training
    data <https://cloud.google.com/vision/automl/docs/prepare>`__ for
    more information.

    CSV file(s) with each line in format:

    ::

        ML_USE,GCS_FILE_PATH,LABEL,LABEL,...

    -  ``ML_USE`` - Identifies the data set that the current row (file)
       applies to. This value can be one of the following:

       -  ``TRAIN`` - Rows in this file are used to train the model.
       -  ``TEST`` - Rows in this file are used to test the model during
          training.
       -  ``UNASSIGNED`` - Rows in this file are not categorized. They
          are Automatically divided into train and test data. 80% for
          training and 20% for testing.

    -  ``GCS_FILE_PATH`` - The Google Cloud Storage location of an image
       of up to 30MB in size. Supported extensions: .JPEG, .GIF, .PNG,
       .WEBP, .BMP, .TIFF, .ICO.

    -  ``LABEL`` - A label that identifies the object in the image.

    For the ``MULTICLASS`` classification type, at most one ``LABEL`` is
    allowed per image. If an image has not yet been labeled, then it
    should be mentioned just once with no ``LABEL``.

    Some sample rows:

    ::

        TRAIN,gs://folder/image1.jpg,daisy
        TEST,gs://folder/image2.jpg,dandelion,tulip,rose
        UNASSIGNED,gs://folder/image3.jpg,daisy
        UNASSIGNED,gs://folder/image4.jpg

    .. raw:: html

        </section><section><h5>Object Detection</h5>
        See [Preparing your training
        data](https://cloud.google.com/vision/automl/object-detection/docs/prepare)
        for more information.

    A CSV file(s) with each line in format:

    ::

        ML_USE,GCS_FILE_PATH,[LABEL],(BOUNDING_BOX | ,,,,,,,)

    -  ``ML_USE`` - Identifies the data set that the current row (file)
       applies to. This value can be one of the following:

       -  ``TRAIN`` - Rows in this file are used to train the model.
       -  ``TEST`` - Rows in this file are used to test the model during
          training.
       -  ``UNASSIGNED`` - Rows in this file are not categorized. They
          are Automatically divided into train and test data. 80% for
          training and 20% for testing.

    -  ``GCS_FILE_PATH`` - The Google Cloud Storage location of an image
       of up to 30MB in size. Supported extensions: .JPEG, .GIF, .PNG.
       Each image is assumed to be exhaustively labeled.

    -  ``LABEL`` - A label that identifies the object in the image
       specified by the ``BOUNDING_BOX``.

    -  ``BOUNDING BOX`` - The vertices of an object in the example
       image. The minimum allowed ``BOUNDING_BOX`` edge length is 0.01,
       and no more than 500 ``BOUNDING_BOX`` instances per image are
       allowed (one ``BOUNDING_BOX`` per line). If an image has no
       looked for objects then it should be mentioned just once with no
       LABEL and the ",,,,,,," in place of the ``BOUNDING_BOX``.

    **Four sample rows:**

    ::

        TRAIN,gs://folder/image1.png,car,0.1,0.1,,,0.3,0.3,,
        TRAIN,gs://folder/image1.png,bike,.7,.6,,,.8,.9,,
        UNASSIGNED,gs://folder/im2.png,car,0.1,0.1,0.2,0.1,0.2,0.3,0.1,0.3
        TEST,gs://folder/im3.png,,,,,,,,,

    .. raw:: html

          </section>
        </div>

    .. raw:: html

        <h4>AutoML Video Intelligence</h4>

    .. raw:: html

        <div class="ds-selector-tabs"><section><h5>Classification</h5>

    See `Preparing your training
    data <https://cloud.google.com/video-intelligence/automl/docs/prepare>`__
    for more information.

    CSV file(s) with each line in format:

    ::

        ML_USE,GCS_FILE_PATH

    For ``ML_USE``, do not use ``VALIDATE``.

    ``GCS_FILE_PATH`` is the path to another .csv file that describes
    training example for a given ``ML_USE``, using the following row
    format:

    ::

        GCS_FILE_PATH,(LABEL,TIME_SEGMENT_START,TIME_SEGMENT_END | ,,)

    Here ``GCS_FILE_PATH`` leads to a video of up to 50GB in size and up
    to 3h duration. Supported extensions: .MOV, .MPEG4, .MP4, .AVI.

    ``TIME_SEGMENT_START`` and ``TIME_SEGMENT_END`` must be within the
    length of the video, and the end time must be after the start time.
    Any segment of a video which has one or more labels on it, is
    considered a hard negative for all other labels. Any segment with no
    labels on it is considered to be unknown. If a whole video is
    unknown, then it should be mentioned just once with ",," in place of
    ``LABEL, TIME_SEGMENT_START,TIME_SEGMENT_END``.

    Sample top level CSV file:

    ::

        TRAIN,gs://folder/train_videos.csv
        TEST,gs://folder/test_videos.csv
        UNASSIGNED,gs://folder/other_videos.csv

    Sample rows of a CSV file for a particular ML_USE:

    ::

        gs://folder/video1.avi,car,120,180.000021
        gs://folder/video1.avi,bike,150,180.000021
        gs://folder/vid2.avi,car,0,60.5
        gs://folder/vid3.avi,,,

    .. raw:: html

        </section><section><h5>Object Tracking</h5>

    See `Preparing your training
    data </video-intelligence/automl/object-tracking/docs/prepare>`__
    for more information.

    CSV file(s) with each line in format:

    ::

        ML_USE,GCS_FILE_PATH

    For ``ML_USE``, do not use ``VALIDATE``.

    ``GCS_FILE_PATH`` is the path to another .csv file that describes
    training example for a given ``ML_USE``, using the following row
    format:

    ::

        GCS_FILE_PATH,LABEL,[INSTANCE_ID],TIMESTAMP,BOUNDING_BOX

    or

    ::

        GCS_FILE_PATH,,,,,,,,,,

    Here ``GCS_FILE_PATH`` leads to a video of up to 50GB in size and up
    to 3h duration. Supported extensions: .MOV, .MPEG4, .MP4, .AVI.
    Providing ``INSTANCE_ID``\ s can help to obtain a better model. When
    a specific labeled entity leaves the video frame, and shows up
    afterwards it is not required, albeit preferable, that the same
    ``INSTANCE_ID`` is given to it.

    ``TIMESTAMP`` must be within the length of the video, the
    ``BOUNDING_BOX`` is assumed to be drawn on the closest video's frame
    to the ``TIMESTAMP``. Any mentioned by the ``TIMESTAMP`` frame is
    expected to be exhaustively labeled and no more than 500
    ``BOUNDING_BOX``-es per frame are allowed. If a whole video is
    unknown, then it should be mentioned just once with ",,,,,,,,,," in
    place of ``LABEL, [INSTANCE_ID],TIMESTAMP,BOUNDING_BOX``.

    Sample top level CSV file:

    ::

         TRAIN,gs://folder/train_videos.csv
         TEST,gs://folder/test_videos.csv
         UNASSIGNED,gs://folder/other_videos.csv

    Seven sample rows of a CSV file for a particular ML_USE:

    ::

         gs://folder/video1.avi,car,1,12.10,0.8,0.8,0.9,0.8,0.9,0.9,0.8,0.9
         gs://folder/video1.avi,car,1,12.90,0.4,0.8,0.5,0.8,0.5,0.9,0.4,0.9
         gs://folder/video1.avi,car,2,12.10,.4,.2,.5,.2,.5,.3,.4,.3
         gs://folder/video1.avi,car,2,12.90,.8,.2,,,.9,.3,,
         gs://folder/video1.avi,bike,,12.50,.45,.45,,,.55,.55,,
         gs://folder/video2.avi,car,1,0,.1,.9,,,.9,.1,,
         gs://folder/video2.avi,,,,,,,,,,,

    .. raw:: html

          </section>
        </div>

    .. raw:: html

        <h4>AutoML Natural Language</h4>

    .. raw:: html

        <div class="ds-selector-tabs"><section><h5>Entity Extraction</h5>

    See `Preparing your training
    data </natural-language/automl/entity-analysis/docs/prepare>`__ for
    more information.

    One or more CSV file(s) with each line in the following format:

    ::

        ML_USE,GCS_FILE_PATH

    -  ``ML_USE`` - Identifies the data set that the current row (file)
       applies to. This value can be one of the following:

       -  ``TRAIN`` - Rows in this file are used to train the model.
       -  ``TEST`` - Rows in this file are used to test the model during
          training.
       -  ``UNASSIGNED`` - Rows in this file are not categorized. They
          are Automatically divided into train and test data. 80% for
          training and 20% for testing..

    -  ``GCS_FILE_PATH`` - a Identifies JSON Lines (.JSONL) file stored
       in Google Cloud Storage that contains in-line text in-line as
       documents for model training.

    After the training data set has been determined from the ``TRAIN``
    and ``UNASSIGNED`` CSV files, the training data is divided into
    train and validation data sets. 70% for training and 30% for
    validation.

    For example:

    ::

        TRAIN,gs://folder/file1.jsonl
        VALIDATE,gs://folder/file2.jsonl
        TEST,gs://folder/file3.jsonl

    **In-line JSONL files**

    In-line .JSONL files contain, per line, a JSON document that wraps a
    [``text_snippet``][google.cloud.automl.v1.TextSnippet] field
    followed by one or more
    [``annotations``][google.cloud.automl.v1.AnnotationPayload] fields,
    which have ``display_name`` and ``text_extraction`` fields to
    describe the entity from the text snippet. Multiple JSON documents
    can be separated using line breaks (\n).

    The supplied text must be annotated exhaustively. For example, if
    you include the text "horse", but do not label it as "animal", then
    "horse" is assumed to not be an "animal".

    Any given text snippet content must have 30,000 characters or less,
    and also be UTF-8 NFC encoded. ASCII is accepted as it is UTF-8 NFC
    encoded.

    For example:

    ::

        {
          "text_snippet": {
            "content": "dog car cat"
          },
          "annotations": [
             {
               "display_name": "animal",
               "text_extraction": {
                 "text_segment": {"start_offset": 0, "end_offset": 2}
              }
             },
             {
              "display_name": "vehicle",
               "text_extraction": {
                 "text_segment": {"start_offset": 4, "end_offset": 6}
               }
             },
             {
               "display_name": "animal",
               "text_extraction": {
                 "text_segment": {"start_offset": 8, "end_offset": 10}
               }
             }
         ]
        }\n
        {
           "text_snippet": {
             "content": "This dog is good."
           },
           "annotations": [
              {
                "display_name": "animal",
                "text_extraction": {
                  "text_segment": {"start_offset": 5, "end_offset": 7}
                }
              }
           ]
        }

    **JSONL files that reference documents**

    .JSONL files contain, per line, a JSON document that wraps a
    ``input_config`` that contains the path to a source document.
    Multiple JSON documents can be separated using line breaks (\n).

    Supported document extensions: .PDF, .TIF, .TIFF

    For example:

    ::

        {
          "document": {
            "input_config": {
              "gcs_source": { "input_uris": [ "gs://folder/document1.pdf" ]
              }
            }
          }
        }\n
        {
          "document": {
            "input_config": {
              "gcs_source": { "input_uris": [ "gs://folder/document2.tif" ]
              }
            }
          }
        }

    **In-line JSONL files with document layout information**

    **Note:** You can only annotate documents using the UI. The format
    described below applies to annotated documents exported using the UI
    or ``exportData``.

    In-line .JSONL files for documents contain, per line, a JSON
    document that wraps a ``document`` field that provides the textual
    content of the document and the layout information.

    For example:

    ::

        {
          "document": {
                  "document_text": {
                    "content": "dog car cat"
                  }
                  "layout": [
                    {
                      "text_segment": {
                        "start_offset": 0,
                        "end_offset": 11,
                       },
                       "page_number": 1,
                       "bounding_poly": {
                          "normalized_vertices": [
                            {"x": 0.1, "y": 0.1},
                            {"x": 0.1, "y": 0.3},
                            {"x": 0.3, "y": 0.3},
                            {"x": 0.3, "y": 0.1},
                          ],
                        },
                        "text_segment_type": TOKEN,
                    }
                  ],
                  "document_dimensions": {
                    "width": 8.27,
                    "height": 11.69,
                    "unit": INCH,
                  }
                  "page_count": 3,
                },
                "annotations": [
                  {
                    "display_name": "animal",
                    "text_extraction": {
                      "text_segment": {"start_offset": 0, "end_offset": 3}
                    }
                  },
                  {
                    "display_name": "vehicle",
                    "text_extraction": {
                      "text_segment": {"start_offset": 4, "end_offset": 7}
                    }
                  },
                  {
                    "display_name": "animal",
                    "text_extraction": {
                      "text_segment": {"start_offset": 8, "end_offset": 11}
                    }
                  },
                ],

    .. raw:: html

        </section><section><h5>Classification</h5>

    See `Preparing your training
    data <https://cloud.google.com/natural-language/automl/docs/prepare>`__
    for more information.

    One or more CSV file(s) with each line in the following format:

    ::

        ML_USE,(TEXT_SNIPPET | GCS_FILE_PATH),LABEL,LABEL,...

    -  ``ML_USE`` - Identifies the data set that the current row (file)
       applies to. This value can be one of the following:

       -  ``TRAIN`` - Rows in this file are used to train the model.
       -  ``TEST`` - Rows in this file are used to test the model during
          training.
       -  ``UNASSIGNED`` - Rows in this file are not categorized. They
          are Automatically divided into train and test data. 80% for
          training and 20% for testing.

    -  ``TEXT_SNIPPET`` and ``GCS_FILE_PATH`` are distinguished by a
       pattern. If the column content is a valid Google Cloud Storage
       file path, that is, prefixed by "gs://", it is treated as a
       ``GCS_FILE_PATH``. Otherwise, if the content is enclosed in
       double quotes (""), it is treated as a ``TEXT_SNIPPET``. For
       ``GCS_FILE_PATH``, the path must lead to a file with supported
       extension and UTF-8 encoding, for example,
       "gs://folder/content.txt" AutoML imports the file content as a
       text snippet. For ``TEXT_SNIPPET``, AutoML imports the column
       content excluding quotes. In both cases, size of the content must
       be 10MB or less in size. For zip files, the size of each file
       inside the zip must be 10MB or less in size.

       For the ``MULTICLASS`` classification type, at most one ``LABEL``
       is allowed.

       The ``ML_USE`` and ``LABEL`` columns are optional. Supported file
       extensions: .TXT, .PDF, .TIF, .TIFF, .ZIP

    A maximum of 100 unique labels are allowed per CSV row.

    Sample rows:

    ::

        TRAIN,"They have bad food and very rude",RudeService,BadFood
        gs://folder/content.txt,SlowService
        TEST,gs://folder/document.pdf
        VALIDATE,gs://folder/text_files.zip,BadFood

    .. raw:: html

        </section><section><h5>Sentiment Analysis</h5>

    See `Preparing your training
    data <https://cloud.google.com/natural-language/automl/docs/prepare>`__
    for more information.

    CSV file(s) with each line in format:

    ::

        ML_USE,(TEXT_SNIPPET | GCS_FILE_PATH),SENTIMENT

    -  ``ML_USE`` - Identifies the data set that the current row (file)
       applies to. This value can be one of the following:

       -  ``TRAIN`` - Rows in this file are used to train the model.
       -  ``TEST`` - Rows in this file are used to test the model during
          training.
       -  ``UNASSIGNED`` - Rows in this file are not categorized. They
          are Automatically divided into train and test data. 80% for
          training and 20% for testing.

    -  ``TEXT_SNIPPET`` and ``GCS_FILE_PATH`` are distinguished by a
       pattern. If the column content is a valid Google Cloud Storage
       file path, that is, prefixed by "gs://", it is treated as a
       ``GCS_FILE_PATH``. Otherwise, if the content is enclosed in
       double quotes (""), it is treated as a ``TEXT_SNIPPET``. For
       ``GCS_FILE_PATH``, the path must lead to a file with supported
       extension and UTF-8 encoding, for example,
       "gs://folder/content.txt" AutoML imports the file content as a
       text snippet. For ``TEXT_SNIPPET``, AutoML imports the column
       content excluding quotes. In both cases, size of the content must
       be 128kB or less in size. For zip files, the size of each file
       inside the zip must be 128kB or less in size.

       The ``ML_USE`` and ``SENTIMENT`` columns are optional. Supported
       file extensions: .TXT, .PDF, .TIF, .TIFF, .ZIP

    -  ``SENTIMENT`` - An integer between 0 and
       Dataset.text_sentiment_dataset_metadata.sentiment_max
       (inclusive). Describes the ordinal of the sentiment - higher
       value means a more positive sentiment. All the values are
       completely relative, i.e. neither 0 needs to mean a negative or
       neutral sentiment nor sentiment_max needs to mean a positive one
       - it is just required that 0 is the least positive sentiment in
       the data, and sentiment_max is the most positive one. The
       SENTIMENT shouldn't be confused with "score" or "magnitude" from
       the previous Natural Language Sentiment Analysis API. All
       SENTIMENT values between 0 and sentiment_max must be represented
       in the imported data. On prediction the same 0 to sentiment_max
       range will be used. The difference between neighboring sentiment
       values needs not to be uniform, e.g. 1 and 2 may be similar
       whereas the difference between 2 and 3 may be large.

    Sample rows:

    ::

        TRAIN,"@freewrytin this is way too good for your product",2
        gs://folder/content.txt,3
        TEST,gs://folder/document.pdf
        VALIDATE,gs://folder/text_files.zip,2

    .. raw:: html

          </section>
        </div>

    .. raw:: html

        <h4>AutoML Tables</h4><div class="ui-datasection-main"><section
        class="selected">

    See `Preparing your training
    data <https://cloud.google.com/automl-tables/docs/prepare>`__ for
    more information.

    You can use either
    [gcs_source][google.cloud.automl.v1.InputConfig.gcs_source] or
    [bigquery_source][google.cloud.automl.v1.InputConfig.bigquery_source].
    All input is concatenated into a single
    [primary_table_spec_id][google.cloud.automl.v1.TablesDatasetMetadata.primary_table_spec_id]

    **For gcs_source:**

    CSV file(s), where the first row of the first file is the header,
    containing unique column names. If the first row of a subsequent
    file is the same as the header, then it is also treated as a header.
    All other rows contain values for the corresponding columns.

    Each .CSV file by itself must be 10GB or smaller, and their total
    size must be 100GB or smaller.

    First three sample rows of a CSV file:

    .. raw:: html

        <pre>
        "Id","First Name","Last Name","Dob","Addresses"
        "1","John","Doe","1968-01-22","[{"status":"current","address":"123_First_Avenue","city":"Seattle","state":"WA","zip":"11111","numberOfYears":"1"},{"status":"previous","address":"456_Main_Street","city":"Portland","state":"OR","zip":"22222","numberOfYears":"5"}]"
        "2","Jane","Doe","1980-10-16","[{"status":"current","address":"789_Any_Avenue","city":"Albany","state":"NY","zip":"33333","numberOfYears":"2"},{"status":"previous","address":"321_Main_Street","city":"Hoboken","state":"NJ","zip":"44444","numberOfYears":"3"}]}
        </pre>

    **For bigquery_source:**

    An URI of a BigQuery table. The user data size of the BigQuery table
    must be 100GB or smaller.

    An imported table must have between 2 and 1,000 columns, inclusive,
    and between 1000 and 100,000,000 rows, inclusive. There are at most
    5 import data running in parallel.

    .. raw:: html

          </section>
        </div>

    **Input field definitions:**

    ``ML_USE`` : ("TRAIN" \| "VALIDATE" \| "TEST" \| "UNASSIGNED")
    Describes how the given example (file) should be used for model
    training. "UNASSIGNED" can be used when user has no preference.

    ``GCS_FILE_PATH`` : The path to a file on Google Cloud Storage. For
    example, "gs://folder/image1.png".

    ``LABEL`` : A display name of an object on an image, video etc.,
    e.g. "dog". Must be up to 32 characters long and can consist only of
    ASCII Latin letters A-Z and a-z, underscores(_), and ASCII digits
    0-9. For each label an AnnotationSpec is created which display_name
    becomes the label; AnnotationSpecs are given back in predictions.

    ``INSTANCE_ID`` : A positive integer that identifies a specific
    instance of a labeled entity on an example. Used e.g. to track two
    cars on a video while being able to tell apart which one is which.

    ``BOUNDING_BOX`` : (``VERTEX,VERTEX,VERTEX,VERTEX`` \|
    ``VERTEX,,,VERTEX,,``) A rectangle parallel to the frame of the
    example (image, video). If 4 vertices are given they are connected
    by edges in the order provided, if 2 are given they are recognized
    as diagonally opposite vertices of the rectangle.

    ``VERTEX`` : (``COORDINATE,COORDINATE``) First coordinate is
    horizontal (x), the second is vertical (y).

    ``COORDINATE`` : A float in 0 to 1 range, relative to total length
    of image or video in given dimension. For fractions the leading
    non-decimal 0 can be omitted (i.e. 0.3 = .3). Point 0,0 is in top
    left.

    ``TIME_SEGMENT_START`` : (``TIME_OFFSET``) Expresses a beginning,
    inclusive, of a time segment within an example that has a time
    dimension (e.g. video).

    ``TIME_SEGMENT_END`` : (``TIME_OFFSET``) Expresses an end,
    exclusive, of a time segment within n example that has a time
    dimension (e.g. video).

    ``TIME_OFFSET`` : A number of seconds as measured from the start of
    an example (e.g. video). Fractions are allowed, up to a microsecond
    precision. "inf" is allowed, and it means the end of the example.

    ``TEXT_SNIPPET`` : The content of a text snippet, UTF-8 encoded,
    enclosed within double quotes ("").

    ``DOCUMENT`` : A field that provides the textual content with
    document and the layout information.

    **Errors:**

    If any of the provided CSV files can't be parsed or if more than
    certain percent of CSV rows cannot be processed then the operation
    fails and nothing is imported. Regardless of overall success or
    failure the per-row failures, up to a certain count cap, is listed
    in Operation.metadata.partial_failures.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_source (google.cloud.automl_v1.types.GcsSource):
            The Google Cloud Storage location for the input content. For
            [AutoMl.ImportData][google.cloud.automl.v1.AutoMl.ImportData],
            ``gcs_source`` points to a CSV file with a structure
            described in
            [InputConfig][google.cloud.automl.v1.InputConfig].

            This field is a member of `oneof`_ ``source``.
        params (MutableMapping[str, str]):
            Additional domain-specific parameters describing the
            semantic of the imported data, any string must be up to
            25000 characters long.

            .. raw:: html

                <h4>AutoML Tables</h4>

            ``schema_inference_version`` : (integer) This value must be
            supplied. The version of the algorithm to use for the
            initial inference of the column data types of the imported
            table. Allowed values: "1".
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="source",
        message="GcsSource",
    )
    params: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )


class BatchPredictInputConfig(proto.Message):
    r"""Input configuration for BatchPredict Action.

    The format of input depends on the ML problem of the model used for
    prediction. As input source the
    [gcs_source][google.cloud.automl.v1.InputConfig.gcs_source] is
    expected, unless specified otherwise.

    The formats are represented in EBNF with commas being literal and
    with non-terminal symbols defined near the end of this comment. The
    formats are:

    .. raw:: html

        <h4>AutoML Vision</h4>
        <div class="ds-selector-tabs"><section><h5>Classification</h5>

    One or more CSV files where each line is a single column:

    ::

        GCS_FILE_PATH

    The Google Cloud Storage location of an image of up to 30MB in size.
    Supported extensions: .JPEG, .GIF, .PNG. This path is treated as the
    ID in the batch predict output.

    Sample rows:

    ::

        gs://folder/image1.jpeg
        gs://folder/image2.gif
        gs://folder/image3.png

    .. raw:: html

        </section><section><h5>Object Detection</h5>

    One or more CSV files where each line is a single column:

    ::

        GCS_FILE_PATH

    The Google Cloud Storage location of an image of up to 30MB in size.
    Supported extensions: .JPEG, .GIF, .PNG. This path is treated as the
    ID in the batch predict output.

    Sample rows:

    ::

        gs://folder/image1.jpeg
        gs://folder/image2.gif
        gs://folder/image3.png

    .. raw:: html

          </section>
        </div>

    .. raw:: html

        <h4>AutoML Video Intelligence</h4>
        <div class="ds-selector-tabs"><section><h5>Classification</h5>

    One or more CSV files where each line is a single column:

    ::

        GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END

    ``GCS_FILE_PATH`` is the Google Cloud Storage location of video up
    to 50GB in size and up to 3h in duration duration. Supported
    extensions: .MOV, .MPEG4, .MP4, .AVI.

    ``TIME_SEGMENT_START`` and ``TIME_SEGMENT_END`` must be within the
    length of the video, and the end time must be after the start time.

    Sample rows:

    ::

        gs://folder/video1.mp4,10,40
        gs://folder/video1.mp4,20,60
        gs://folder/vid2.mov,0,inf

    .. raw:: html

        </section><section><h5>Object Tracking</h5>

    One or more CSV files where each line is a single column:

    ::

        GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END

    ``GCS_FILE_PATH`` is the Google Cloud Storage location of video up
    to 50GB in size and up to 3h in duration duration. Supported
    extensions: .MOV, .MPEG4, .MP4, .AVI.

    ``TIME_SEGMENT_START`` and ``TIME_SEGMENT_END`` must be within the
    length of the video, and the end time must be after the start time.

    Sample rows:

    ::

        gs://folder/video1.mp4,10,40
        gs://folder/video1.mp4,20,60
        gs://folder/vid2.mov,0,inf

    .. raw:: html

          </section>
        </div>

    .. raw:: html

        <h4>AutoML Natural Language</h4>
        <div class="ds-selector-tabs"><section><h5>Classification</h5>

    One or more CSV files where each line is a single column:

    ::

        GCS_FILE_PATH

    ``GCS_FILE_PATH`` is the Google Cloud Storage location of a text
    file. Supported file extensions: .TXT, .PDF, .TIF, .TIFF

    Text files can be no larger than 10MB in size.

    Sample rows:

    ::

        gs://folder/text1.txt
        gs://folder/text2.pdf
        gs://folder/text3.tif

    .. raw:: html

        </section><section><h5>Sentiment Analysis</h5>
        One or more CSV files where each line is a single column:

    ::

        GCS_FILE_PATH

    ``GCS_FILE_PATH`` is the Google Cloud Storage location of a text
    file. Supported file extensions: .TXT, .PDF, .TIF, .TIFF

    Text files can be no larger than 128kB in size.

    Sample rows:

    ::

        gs://folder/text1.txt
        gs://folder/text2.pdf
        gs://folder/text3.tif

    .. raw:: html

        </section><section><h5>Entity Extraction</h5>

    One or more JSONL (JSON Lines) files that either provide inline text
    or documents. You can only use one format, either inline text or
    documents, for a single call to [AutoMl.BatchPredict].

    Each JSONL file contains a per line a proto that wraps a temporary
    user-assigned TextSnippet ID (string up to 2000 characters long)
    called "id", a TextSnippet proto (in JSON representation) and zero
    or more TextFeature protos. Any given text snippet content must have
    30,000 characters or less, and also be UTF-8 NFC encoded (ASCII
    already is). The IDs provided should be unique.

    Each document JSONL file contains, per line, a proto that wraps a
    Document proto with ``input_config`` set. Each document cannot
    exceed 2MB in size.

    Supported document extensions: .PDF, .TIF, .TIFF

    Each JSONL file must not exceed 100MB in size, and no more than 20
    JSONL files may be passed.

    Sample inline JSONL file (Shown with artificial line breaks. Actual
    line breaks are denoted by "\n".):

    ::

        {
           "id": "my_first_id",
           "text_snippet": { "content": "dog car cat"},
           "text_features": [
             {
               "text_segment": {"start_offset": 4, "end_offset": 6},
               "structural_type": PARAGRAPH,
               "bounding_poly": {
                 "normalized_vertices": [
                   {"x": 0.1, "y": 0.1},
                   {"x": 0.1, "y": 0.3},
                   {"x": 0.3, "y": 0.3},
                   {"x": 0.3, "y": 0.1},
                 ]
               },
             }
           ],
         }\n
         {
           "id": "2",
           "text_snippet": {
             "content": "Extended sample content",
             "mime_type": "text/plain"
           }
         }

    Sample document JSONL file (Shown with artificial line breaks.
    Actual line breaks are denoted by "\n".):

    ::

         {
           "document": {
             "input_config": {
               "gcs_source": { "input_uris": [ "gs://folder/document1.pdf" ]
               }
             }
           }
         }\n
         {
           "document": {
             "input_config": {
               "gcs_source": { "input_uris": [ "gs://folder/document2.tif" ]
               }
             }
           }
         }

    .. raw:: html

          </section>
        </div>

    .. raw:: html

        <h4>AutoML Tables</h4><div class="ui-datasection-main"><section
        class="selected">

    See `Preparing your training
    data <https://cloud.google.com/automl-tables/docs/predict-batch>`__
    for more information.

    You can use either
    [gcs_source][google.cloud.automl.v1.BatchPredictInputConfig.gcs_source]
    or [bigquery_source][BatchPredictInputConfig.bigquery_source].

    **For gcs_source:**

    CSV file(s), each by itself 10GB or smaller and total size must be
    100GB or smaller, where first file must have a header containing
    column names. If the first row of a subsequent file is the same as
    the header, then it is also treated as a header. All other rows
    contain values for the corresponding columns.

    The column names must contain the model's
    [input_feature_column_specs'][google.cloud.automl.v1.TablesModelMetadata.input_feature_column_specs]
    [display_name-s][google.cloud.automl.v1.ColumnSpec.display_name]
    (order doesn't matter). The columns corresponding to the model's
    input feature column specs must contain values compatible with the
    column spec's data types. Prediction on all the rows, i.e. the CSV
    lines, will be attempted.

    Sample rows from a CSV file:

    .. raw:: html

        <pre>
        "First Name","Last Name","Dob","Addresses"
        "John","Doe","1968-01-22","[{"status":"current","address":"123_First_Avenue","city":"Seattle","state":"WA","zip":"11111","numberOfYears":"1"},{"status":"previous","address":"456_Main_Street","city":"Portland","state":"OR","zip":"22222","numberOfYears":"5"}]"
        "Jane","Doe","1980-10-16","[{"status":"current","address":"789_Any_Avenue","city":"Albany","state":"NY","zip":"33333","numberOfYears":"2"},{"status":"previous","address":"321_Main_Street","city":"Hoboken","state":"NJ","zip":"44444","numberOfYears":"3"}]}
        </pre>

    **For bigquery_source:**

    The URI of a BigQuery table. The user data size of the BigQuery
    table must be 100GB or smaller.

    The column names must contain the model's
    [input_feature_column_specs'][google.cloud.automl.v1.TablesModelMetadata.input_feature_column_specs]
    [display_name-s][google.cloud.automl.v1.ColumnSpec.display_name]
    (order doesn't matter). The columns corresponding to the model's
    input feature column specs must contain values compatible with the
    column spec's data types. Prediction on all the rows of the table
    will be attempted.

    .. raw:: html

          </section>
        </div>

    **Input field definitions:**

    ``GCS_FILE_PATH`` : The path to a file on Google Cloud Storage. For
    example, "gs://folder/video.avi".

    ``TIME_SEGMENT_START`` : (``TIME_OFFSET``) Expresses a beginning,
    inclusive, of a time segment within an example that has a time
    dimension (e.g. video).

    ``TIME_SEGMENT_END`` : (``TIME_OFFSET``) Expresses an end,
    exclusive, of a time segment within n example that has a time
    dimension (e.g. video).

    ``TIME_OFFSET`` : A number of seconds as measured from the start of
    an example (e.g. video). Fractions are allowed, up to a microsecond
    precision. "inf" is allowed, and it means the end of the example.

    **Errors:**

    If any of the provided CSV files can't be parsed or if more than
    certain percent of CSV rows cannot be processed then the operation
    fails and prediction does not happen. Regardless of overall success
    or failure the per-row failures, up to a certain count cap, will be
    listed in Operation.metadata.partial_failures.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_source (google.cloud.automl_v1.types.GcsSource):
            Required. The Google Cloud Storage location
            for the input content.

            This field is a member of `oneof`_ ``source``.
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="source",
        message="GcsSource",
    )


class DocumentInputConfig(proto.Message):
    r"""Input configuration of a
    [Document][google.cloud.automl.v1.Document].

    Attributes:
        gcs_source (google.cloud.automl_v1.types.GcsSource):
            The Google Cloud Storage location of the
            document file. Only a single path should be
            given.

            Max supported size: 512MB.

            Supported extensions: .PDF.
    """

    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GcsSource",
    )


class OutputConfig(proto.Message):
    r"""-  For Translation: CSV file ``translation.csv``, with each line in
       format: ML_USE,GCS_FILE_PATH GCS_FILE_PATH leads to a .TSV file
       which describes examples that have given ML_USE, using the
       following row format per line: TEXT_SNIPPET (in source language)
       \\t TEXT_SNIPPET (in target language)

       -  For Tables: Output depends on whether the dataset was imported
          from Google Cloud Storage or BigQuery. Google Cloud Storage
          case:
          [gcs_destination][google.cloud.automl.v1p1beta.OutputConfig.gcs_destination]
          must be set. Exported are CSV file(s) ``tables_1.csv``,
          ``tables_2.csv``,...,\ ``tables_N.csv`` with each having as
          header line the table's column names, and all other lines
          contain values for the header columns. BigQuery case:
          [bigquery_destination][google.cloud.automl.v1p1beta.OutputConfig.bigquery_destination]
          pointing to a BigQuery project must be set. In the given
          project a new dataset will be created with name
          ``export_data_<automl-dataset-display-name>_<timestamp-of-export-call>``
          where will be made BigQuery-dataset-name compatible (e.g. most
          special characters will become underscores), and timestamp
          will be in YYYY_MM_DDThh_mm_ss_sssZ "based on ISO-8601"
          format. In that dataset a new table called ``primary_table``
          will be created, and filled with precisely the same data as
          this obtained on import.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_destination (google.cloud.automl_v1.types.GcsDestination):
            Required. The Google Cloud Storage location where the output
            is to be written to. For Image Object Detection, Text
            Extraction, Video Classification and Tables, in the given
            directory a new directory will be created with name:
            export_data-- where timestamp is in YYYY-MM-DDThh:mm:ss.sssZ
            ISO-8601 format. All export output will be written into that
            directory.

            This field is a member of `oneof`_ ``destination``.
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="destination",
        message="GcsDestination",
    )


class BatchPredictOutputConfig(proto.Message):
    r"""Output configuration for BatchPredict Action.

    As destination the
    [gcs_destination][google.cloud.automl.v1.BatchPredictOutputConfig.gcs_destination]
    must be set unless specified otherwise for a domain. If
    gcs_destination is set then in the given directory a new directory
    is created. Its name will be "prediction--", where timestamp is in
    YYYY-MM-DDThh:mm:ss.sssZ ISO-8601 format. The contents of it depends
    on the ML problem the predictions are made for.

    -  For Image Classification: In the created directory files
       ``image_classification_1.jsonl``,
       ``image_classification_2.jsonl``,...,\ ``image_classification_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of the successfully predicted images and annotations. A
       single image will be listed only once with all its annotations,
       and its annotations will never be split across files. Each .JSONL
       file will contain, per line, a JSON representation of a proto
       that wraps image's "ID" : "<id_value>" followed by a list of zero
       or more AnnotationPayload protos (called annotations), which have
       classification detail populated. If prediction for any image
       failed (partially or completely), then an additional
       ``errors_1.jsonl``, ``errors_2.jsonl``,..., ``errors_N.jsonl``
       files will be created (N depends on total number of failed
       predictions). These files will have a JSON representation of a
       proto that wraps the same "ID" : "<id_value>" but here followed
       by exactly one
       ```google.rpc.Status`` <https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto>`__
       containing only ``code`` and ``message``\ fields.

    -  For Image Object Detection: In the created directory files
       ``image_object_detection_1.jsonl``,
       ``image_object_detection_2.jsonl``,...,\ ``image_object_detection_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of the successfully predicted images and annotations. Each
       .JSONL file will contain, per line, a JSON representation of a
       proto that wraps image's "ID" : "<id_value>" followed by a list
       of zero or more AnnotationPayload protos (called annotations),
       which have image_object_detection detail populated. A single
       image will be listed only once with all its annotations, and its
       annotations will never be split across files. If prediction for
       any image failed (partially or completely), then additional
       ``errors_1.jsonl``, ``errors_2.jsonl``,..., ``errors_N.jsonl``
       files will be created (N depends on total number of failed
       predictions). These files will have a JSON representation of a
       proto that wraps the same "ID" : "<id_value>" but here followed
       by exactly one
       ```google.rpc.Status`` <https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto>`__
       containing only ``code`` and ``message``\ fields.

    -  For Video Classification: In the created directory a
       video_classification.csv file, and a .JSON file per each video
       classification requested in the input (i.e. each line in given
       CSV(s)), will be created.

       ::

          The format of video_classification.csv is:
          GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END,JSON_FILE_NAME,STATUS
          where:
          GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END = matches 1 to 1
              the prediction input lines (i.e. video_classification.csv has
              precisely the same number of lines as the prediction input had.)
          JSON_FILE_NAME = Name of .JSON file in the output directory, which
              contains prediction responses for the video time segment.
          STATUS = "OK" if prediction completed successfully, or an error code
              with message otherwise. If STATUS is not "OK" then the .JSON file
              for that line may not exist or be empty.

          Each .JSON file, assuming STATUS is "OK", will contain a list of
          AnnotationPayload protos in JSON format, which are the predictions
          for the video time segment the file is assigned to in the
          video_classification.csv. All AnnotationPayload protos will have
          video_classification field set, and will be sorted by
          video_classification.type field (note that the returned types are
          governed by `classifaction_types` parameter in
          [PredictService.BatchPredictRequest.params][]).

    -  For Video Object Tracking: In the created directory a
       video_object_tracking.csv file will be created, and multiple
       files video_object_trackinng_1.json,
       video_object_trackinng_2.json,..., video_object_trackinng_N.json,
       where N is the number of requests in the input (i.e. the number
       of lines in given CSV(s)).

       ::

          The format of video_object_tracking.csv is:
          GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END,JSON_FILE_NAME,STATUS
          where:
          GCS_FILE_PATH,TIME_SEGMENT_START,TIME_SEGMENT_END = matches 1 to 1
              the prediction input lines (i.e. video_object_tracking.csv has
              precisely the same number of lines as the prediction input had.)
          JSON_FILE_NAME = Name of .JSON file in the output directory, which
              contains prediction responses for the video time segment.
          STATUS = "OK" if prediction completed successfully, or an error
              code with message otherwise. If STATUS is not "OK" then the .JSON
              file for that line may not exist or be empty.

          Each .JSON file, assuming STATUS is "OK", will contain a list of
          AnnotationPayload protos in JSON format, which are the predictions
          for each frame of the video time segment the file is assigned to in
          video_object_tracking.csv. All AnnotationPayload protos will have
          video_object_tracking field set.

    -  For Text Classification: In the created directory files
       ``text_classification_1.jsonl``,
       ``text_classification_2.jsonl``,...,\ ``text_classification_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of inputs and annotations found.

       ::

          Each .JSONL file will contain, per line, a JSON representation of a
          proto that wraps input text file (or document) in
          the text snippet (or document) proto and a list of
          zero or more AnnotationPayload protos (called annotations), which
          have classification detail populated. A single text file (or
          document) will be listed only once with all its annotations, and its
          annotations will never be split across files.

          If prediction for any input file (or document) failed (partially or
          completely), then additional `errors_1.jsonl`, `errors_2.jsonl`,...,
          `errors_N.jsonl` files will be created (N depends on total number of
          failed predictions). These files will have a JSON representation of a
          proto that wraps input file followed by exactly one
          [`google.rpc.Status`](https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
          containing only `code` and `message`.

    -  For Text Sentiment: In the created directory files
       ``text_sentiment_1.jsonl``,
       ``text_sentiment_2.jsonl``,...,\ ``text_sentiment_N.jsonl`` will
       be created, where N may be 1, and depends on the total number of
       inputs and annotations found.

       ::

          Each .JSONL file will contain, per line, a JSON representation of a
          proto that wraps input text file (or document) in
          the text snippet (or document) proto and a list of
          zero or more AnnotationPayload protos (called annotations), which
          have text_sentiment detail populated. A single text file (or
          document) will be listed only once with all its annotations, and its
          annotations will never be split across files.

          If prediction for any input file (or document) failed (partially or
          completely), then additional `errors_1.jsonl`, `errors_2.jsonl`,...,
          `errors_N.jsonl` files will be created (N depends on total number of
          failed predictions). These files will have a JSON representation of a
          proto that wraps input file followed by exactly one
          [`google.rpc.Status`](https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto)
          containing only `code` and `message`.

    -  For Text Extraction: In the created directory files
       ``text_extraction_1.jsonl``,
       ``text_extraction_2.jsonl``,...,\ ``text_extraction_N.jsonl``
       will be created, where N may be 1, and depends on the total
       number of inputs and annotations found. The contents of these
       .JSONL file(s) depend on whether the input used inline text, or
       documents. If input was inline, then each .JSONL file will
       contain, per line, a JSON representation of a proto that wraps
       given in request text snippet's "id" (if specified), followed by
       input text snippet, and a list of zero or more AnnotationPayload
       protos (called annotations), which have text_extraction detail
       populated. A single text snippet will be listed only once with
       all its annotations, and its annotations will never be split
       across files. If input used documents, then each .JSONL file will
       contain, per line, a JSON representation of a proto that wraps
       given in request document proto, followed by its OCR-ed
       representation in the form of a text snippet, finally followed by
       a list of zero or more AnnotationPayload protos (called
       annotations), which have text_extraction detail populated and
       refer, via their indices, to the OCR-ed text snippet. A single
       document (and its text snippet) will be listed only once with all
       its annotations, and its annotations will never be split across
       files. If prediction for any text snippet failed (partially or
       completely), then additional ``errors_1.jsonl``,
       ``errors_2.jsonl``,..., ``errors_N.jsonl`` files will be created
       (N depends on total number of failed predictions). These files
       will have a JSON representation of a proto that wraps either the
       "id" : "<id_value>" (in case of inline) or the document proto (in
       case of document) but here followed by exactly one
       ```google.rpc.Status`` <https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto>`__
       containing only ``code`` and ``message``.

    -  For Tables: Output depends on whether
       [gcs_destination][google.cloud.automl.v1p1beta.BatchPredictOutputConfig.gcs_destination]
       or
       [bigquery_destination][google.cloud.automl.v1p1beta.BatchPredictOutputConfig.bigquery_destination]
       is set (either is allowed). Google Cloud Storage case: In the
       created directory files ``tables_1.csv``, ``tables_2.csv``,...,
       ``tables_N.csv`` will be created, where N may be 1, and depends
       on the total number of the successfully predicted rows. For all
       CLASSIFICATION
       [prediction_type-s][google.cloud.automl.v1p1beta.TablesModelMetadata.prediction_type]:
       Each .csv file will contain a header, listing all columns'
       [display_name-s][google.cloud.automl.v1p1beta.ColumnSpec.display_name]
       given on input followed by M target column names in the format of
       "<[target_column_specs][google.cloud.automl.v1p1beta.TablesModelMetadata.target_column_spec]
       [display_name][google.cloud.automl.v1p1beta.ColumnSpec.display_name]>*\ score"
       where M is the number of distinct target values, i.e. number of
       distinct values in the target column of the table used to train
       the model. Subsequent lines will contain the respective values of
       successfully predicted rows, with the last, i.e. the target,
       columns having the corresponding prediction
       [scores][google.cloud.automl.v1p1beta.TablesAnnotation.score].
       For REGRESSION and FORECASTING
       [prediction_type-s][google.cloud.automl.v1p1beta.TablesModelMetadata.prediction_type]:
       Each .csv file will contain a header, listing all columns'
       [display_name-s][google.cloud.automl.v1p1beta.display_name] given
       on input followed by the predicted target column with name in the
       format of
       "predicted\ <[target_column_specs][google.cloud.automl.v1p1beta.TablesModelMetadata.target_column_spec]
       [display_name][google.cloud.automl.v1p1beta.ColumnSpec.display_name]>"
       Subsequent lines will contain the respective values of
       successfully predicted rows, with the last, i.e. the target,
       column having the predicted target value. If prediction for any
       rows failed, then an additional ``errors_1.csv``,
       ``errors_2.csv``,..., ``errors_N.csv`` will be created (N depends
       on total number of failed rows). These files will have analogous
       format as ``tables_*.csv``, but always with a single target
       column
       having*\ ```google.rpc.Status`` <https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto>`__\ *represented
       as a JSON string, and containing only ``code`` and ``message``.
       BigQuery case:
       [bigquery_destination][google.cloud.automl.v1p1beta.OutputConfig.bigquery_destination]
       pointing to a BigQuery project must be set. In the given project
       a new dataset will be created with name
       ``prediction_<model-display-name>_<timestamp-of-prediction-call>``
       where will be made BigQuery-dataset-name compatible (e.g. most
       special characters will become underscores), and timestamp will
       be in YYYY_MM_DDThh_mm_ss_sssZ "based on ISO-8601" format. In the
       dataset two tables will be created, ``predictions``, and
       ``errors``. The ``predictions`` table's column names will be the
       input columns'
       [display_name-s][google.cloud.automl.v1p1beta.ColumnSpec.display_name]
       followed by the target column with name in the format of
       "predicted*\ <[target_column_specs][google.cloud.automl.v1p1beta.TablesModelMetadata.target_column_spec]
       [display_name][google.cloud.automl.v1p1beta.ColumnSpec.display_name]>"
       The input feature columns will contain the respective values of
       successfully predicted rows, with the target column having an
       ARRAY of
       [AnnotationPayloads][google.cloud.automl.v1p1beta.AnnotationPayload],
       represented as STRUCT-s, containing
       [TablesAnnotation][google.cloud.automl.v1p1beta.TablesAnnotation].
       The ``errors`` table contains rows for which the prediction has
       failed, it has analogous input columns while the target column
       name is in the format of
       "errors_<[target_column_specs][google.cloud.automl.v1p1beta.TablesModelMetadata.target_column_spec]
       [display_name][google.cloud.automl.v1p1beta.ColumnSpec.display_name]>",
       and as a value has
       ```google.rpc.Status`` <https://github.com/googleapis/googleapis/blob/master/google/rpc/status.proto>`__
       represented as a STRUCT, and containing only ``code`` and
       ``message``.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_destination (google.cloud.automl_v1.types.GcsDestination):
            Required. The Google Cloud Storage location
            of the directory where the output is to be
            written to.

            This field is a member of `oneof`_ ``destination``.
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="destination",
        message="GcsDestination",
    )


class ModelExportOutputConfig(proto.Message):
    r"""Output configuration for ModelExport Action.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_destination (google.cloud.automl_v1.types.GcsDestination):
            Required. The Google Cloud Storage location where the model
            is to be written to. This location may only be set for the
            following model formats: "tflite", "edgetpu_tflite",
            "tf_saved_model", "tf_js", "core_ml".

            Under the directory given as the destination a new one with
            name "model-export--", where timestamp is in
            YYYY-MM-DDThh:mm:ss.sssZ ISO-8601 format, will be created.
            Inside the model and any of its supporting files will be
            written.

            This field is a member of `oneof`_ ``destination``.
        model_format (str):
            The format in which the model must be exported. The
            available, and default, formats depend on the problem and
            model type (if given problem and type combination doesn't
            have a format listed, it means its models are not
            exportable):

            -  For Image Classification mobile-low-latency-1,
               mobile-versatile-1, mobile-high-accuracy-1: "tflite"
               (default), "edgetpu_tflite", "tf_saved_model", "tf_js",
               "docker".

            -  For Image Classification mobile-core-ml-low-latency-1,
               mobile-core-ml-versatile-1,
               mobile-core-ml-high-accuracy-1: "core_ml" (default).

            -  For Image Object Detection mobile-low-latency-1,
               mobile-versatile-1, mobile-high-accuracy-1: "tflite",
               "tf_saved_model", "tf_js". Formats description:

            -  tflite - Used for Android mobile devices.

            -  edgetpu_tflite - Used for `Edge
               TPU <https://cloud.google.com/edge-tpu/>`__ devices.

            -  tf_saved_model - A tensorflow model in SavedModel format.

            -  tf_js - A
               `TensorFlow.js <https://www.tensorflow.org/js>`__ model
               that can be used in the browser and in Node.js using
               JavaScript.

            -  docker - Used for Docker containers. Use the params field
               to customize the container. The container is verified to
               work correctly on ubuntu 16.04 operating system. See more
               at `containers
               quickstart <https://cloud.google.com/vision/automl/docs/containers-gcs-quickstart>`__

            -  core_ml - Used for iOS mobile devices.
        params (MutableMapping[str, str]):
            Additional model-type and format specific parameters
            describing the requirements for the to be exported model
            files, any string must be up to 25000 characters long.

            -  For ``docker`` format: ``cpu_architecture`` - (string)
               "x86_64" (default). ``gpu_architecture`` - (string)
               "none" (default), "nvidia".
    """

    gcs_destination: "GcsDestination" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="destination",
        message="GcsDestination",
    )
    model_format: str = proto.Field(
        proto.STRING,
        number=4,
    )
    params: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=2,
    )


class GcsSource(proto.Message):
    r"""The Google Cloud Storage location for the input content.

    Attributes:
        input_uris (MutableSequence[str]):
            Required. Google Cloud Storage URIs to input files, up to
            2000 characters long. Accepted forms:

            -  Full object path, e.g. gs://bucket/directory/object.csv
    """

    input_uris: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )


class GcsDestination(proto.Message):
    r"""The Google Cloud Storage location where the output is to be
    written to.

    Attributes:
        output_uri_prefix (str):
            Required. Google Cloud Storage URI to output directory, up
            to 2000 characters long. Accepted forms:

            -  Prefix path: gs://bucket/directory The requesting user
               must have write permission to the bucket. The directory
               is created if it doesn't exist.
    """

    output_uri_prefix: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unit tests."""

from google.api_core import exceptions
from google.auth.credentials import AnonymousCredentials
from google.protobuf import struct_pb2 as struct
import mock
import pandas
import pytest

from google.cloud import automl_v1beta1
from google.cloud.automl_v1beta1.types import data_items, data_types

PROJECT = "project"
REGION = "region"
LOCATION_PATH = "projects/{}/locations/{}".format(PROJECT, REGION)


class TestTablesClient(object):
    def tables_client(
        self, client_attrs={}, prediction_client_attrs={}, gcs_client_attrs={}
    ):
        client_mock = mock.Mock(**client_attrs)
        prediction_client_mock = mock.Mock(**prediction_client_attrs)
        gcs_client_mock = mock.Mock(**gcs_client_attrs)
        return automl_v1beta1.TablesClient(
            client=client_mock,
            prediction_client=prediction_client_mock,
            gcs_client=gcs_client_mock,
            project=PROJECT,
            region=REGION,
        )

    def test_list_datasets_empty(self):
        client = self.tables_client(
            client_attrs={
                "list_datasets.return_value": [],
                "location_path.return_value": LOCATION_PATH,
            },
            prediction_client_attrs={},
        )
        ds = client.list_datasets()

        request = automl_v1beta1.ListDatasetsRequest(parent=LOCATION_PATH)
        client.auto_ml_client.list_datasets.assert_called_with(request=request)
        assert ds == []

    def test_list_datasets_not_empty(self):
        datasets = ["some_dataset"]
        client = self.tables_client(
            client_attrs={
                "list_datasets.return_value": datasets,
                "location_path.return_value": LOCATION_PATH,
            },
            prediction_client_attrs={},
        )
        ds = client.list_datasets()

        request = automl_v1beta1.ListDatasetsRequest(parent=LOCATION_PATH)
        client.auto_ml_client.list_datasets.assert_called_with(request=request)
        assert len(ds) == 1
        assert ds[0] == "some_dataset"

    def test_get_dataset_no_value(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.get_dataset()
        client.auto_ml_client.get_dataset.assert_not_called()

    def test_get_dataset_name(self):
        dataset_actual = "dataset"
        client = self.tables_client({"get_dataset.return_value": dataset_actual}, {})
        dataset = client.get_dataset(dataset_name="my_dataset")
        client.auto_ml_client.get_dataset.assert_called_with(
            request=automl_v1beta1.GetDatasetRequest(name="my_dataset")
        )
        assert dataset == dataset_actual

    def test_get_no_dataset(self):
        client = self.tables_client(
            {"get_dataset.side_effect": exceptions.NotFound("err")}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.get_dataset(dataset_name="my_dataset")
        client.auto_ml_client.get_dataset.assert_called_with(
            request=automl_v1beta1.GetDatasetRequest(name="my_dataset")
        )

    def test_get_dataset_from_empty_list(self):
        client = self.tables_client({"list_datasets.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.get_dataset(dataset_display_name="my_dataset")

    def test_get_dataset_from_list_not_found(self):
        client = self.tables_client(
            {"list_datasets.return_value": [mock.Mock(display_name="not_it")]}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.get_dataset(dataset_display_name="my_dataset")

    def test_get_dataset_from_list(self):
        client = self.tables_client(
            {
                "list_datasets.return_value": [
                    mock.Mock(display_name="not_it"),
                    mock.Mock(display_name="my_dataset"),
                ]
            },
            {},
        )
        dataset = client.get_dataset(dataset_display_name="my_dataset")
        assert dataset.display_name == "my_dataset"

    def test_get_dataset_from_list_ambiguous(self):
        client = self.tables_client(
            {
                "list_datasets.return_value": [
                    mock.Mock(display_name="my_dataset"),
                    mock.Mock(display_name="not_my_dataset"),
                    mock.Mock(display_name="my_dataset"),
                ]
            },
            {},
        )
        with pytest.raises(ValueError):
            client.get_dataset(dataset_display_name="my_dataset")

    def test_create_dataset(self):
        client = self.tables_client(
            {
                "location_path.return_value": LOCATION_PATH,
                "create_dataset.return_value": mock.Mock(display_name="name"),
            },
            {},
        )
        metadata = {"primary_table_spec_id": "1234"}
        dataset = client.create_dataset("name", metadata=metadata)

        client.auto_ml_client.create_dataset.assert_called_with(
            request=automl_v1beta1.CreateDatasetRequest(
                parent=LOCATION_PATH,
                dataset={"display_name": "name", "tables_dataset_metadata": metadata},
            )
        )
        assert dataset.display_name == "name"

    def test_delete_dataset(self):
        dataset = mock.Mock()
        dataset.configure_mock(name="name")
        client = self.tables_client({"delete_dataset.return_value": None}, {})
        client.delete_dataset(dataset=dataset)
        client.auto_ml_client.delete_dataset.assert_called_with(
            request=automl_v1beta1.DeleteDatasetRequest(name="name")
        )

    def test_delete_dataset_not_found(self):
        client = self.tables_client({"list_datasets.return_value": []}, {})
        client.delete_dataset(dataset_display_name="not_found")
        client.auto_ml_client.delete_dataset.assert_not_called()

    def test_delete_dataset_name(self):
        client = self.tables_client({"delete_dataset.return_value": None}, {})
        client.delete_dataset(dataset_name="name")
        client.auto_ml_client.delete_dataset.assert_called_with(
            request=automl_v1beta1.DeleteDatasetRequest(name="name")
        )

    def test_export_not_found(self):
        client = self.tables_client({"list_datasets.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.export_data(dataset_display_name="name", gcs_input_uris="uri")

        client.auto_ml_client.export_data.assert_not_called()

    def test_export_gcs_uri(self):
        client = self.tables_client({"export_data.return_value": None}, {})
        client.export_data(dataset_name="name", gcs_output_uri_prefix="uri")
        client.auto_ml_client.export_data.assert_called_with(
            request=automl_v1beta1.ExportDataRequest(
                name="name",
                output_config={"gcs_destination": {"output_uri_prefix": "uri"}},
            )
        )

    def test_export_bq_uri(self):
        client = self.tables_client({"export_data.return_value": None}, {})
        client.export_data(dataset_name="name", bigquery_output_uri="uri")
        client.auto_ml_client.export_data.assert_called_with(
            request=automl_v1beta1.ExportDataRequest(
                name="name",
                output_config={"bigquery_destination": {"output_uri": "uri"}},
            )
        )

    def test_import_not_found(self):
        client = self.tables_client({"list_datasets.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.import_data(dataset_display_name="name", gcs_input_uris="uri")

        client.auto_ml_client.import_data.assert_not_called()

    def test_import_pandas_dataframe(self):
        client = self.tables_client(
            gcs_client_attrs={
                "bucket_name": "my_bucket",
                "upload_pandas_dataframe.return_value": "uri",
            }
        )
        dataframe = pandas.DataFrame({})
        client.import_data(
            project=PROJECT,
            region=REGION,
            dataset_name="name",
            pandas_dataframe=dataframe,
        )
        client.gcs_client.ensure_bucket_exists.assert_called_with(PROJECT, REGION)
        client.gcs_client.upload_pandas_dataframe.assert_called_with(dataframe)
        client.auto_ml_client.import_data.assert_called_with(
            request=automl_v1beta1.ImportDataRequest(
                name="name", input_config={"gcs_source": {"input_uris": ["uri"]}}
            )
        )

    def test_import_pandas_dataframe_init_gcs(self):
        client = automl_v1beta1.TablesClient(
            client=mock.Mock(),
            prediction_client=mock.Mock(),
            project=PROJECT,
            region=REGION,
            credentials=AnonymousCredentials(),
        )

        dataframe = pandas.DataFrame({})
        patch = mock.patch(
            "google.cloud.automl_v1beta1.services.tables.tables_client.gcs_client.GcsClient",
            bucket_name="my_bucket",
        )
        with patch as MockGcsClient:
            mockInstance = MockGcsClient.return_value
            mockInstance.upload_pandas_dataframe.return_value = "uri"

            client.import_data(dataset_name="name", pandas_dataframe=dataframe)

            assert client.gcs_client is mockInstance
            client.gcs_client.ensure_bucket_exists.assert_called_with(PROJECT, REGION)
            client.gcs_client.upload_pandas_dataframe.assert_called_with(dataframe)
            client.auto_ml_client.import_data.assert_called_with(
                request=automl_v1beta1.ImportDataRequest(
                    name="name", input_config={"gcs_source": {"input_uris": ["uri"]}}
                )
            )

    def test_import_gcs_uri(self):
        client = self.tables_client({"import_data.return_value": None}, {})
        client.import_data(dataset_name="name", gcs_input_uris="uri")
        client.auto_ml_client.import_data.assert_called_with(
            request=automl_v1beta1.ImportDataRequest(
                name="name", input_config={"gcs_source": {"input_uris": ["uri"]}}
            )
        )

    def test_import_gcs_uris(self):
        client = self.tables_client({"import_data.return_value": None}, {})
        client.import_data(dataset_name="name", gcs_input_uris=["uri", "uri"])
        client.auto_ml_client.import_data.assert_called_with(
            request=automl_v1beta1.ImportDataRequest(
                name="name", input_config={"gcs_source": {"input_uris": ["uri", "uri"]}}
            )
        )

    def test_import_bq_uri(self):
        client = self.tables_client({"import_data.return_value": None}, {})
        client.import_data(dataset_name="name", bigquery_input_uri="uri")
        client.auto_ml_client.import_data.assert_called_with(
            request=automl_v1beta1.ImportDataRequest(
                name="name", input_config={"bigquery_source": {"input_uri": "uri"}}
            )
        )

    def test_list_table_specs(self):
        client = self.tables_client({"list_table_specs.return_value": None}, {})
        client.list_table_specs(dataset_name="name")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )

    def test_list_table_specs_not_found(self):
        client = self.tables_client(
            {"list_table_specs.side_effect": exceptions.NotFound("not found")}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.list_table_specs(dataset_name="name")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )

    def test_get_table_spec(self):
        client = self.tables_client({}, {})
        client.get_table_spec("name")
        client.auto_ml_client.get_table_spec.assert_called_with(
            request=automl_v1beta1.GetTableSpecRequest(name="name")
        )

    def test_get_column_spec(self):
        client = self.tables_client({}, {})
        client.get_column_spec("name")
        client.auto_ml_client.get_column_spec.assert_called_with(
            request=automl_v1beta1.GetColumnSpecRequest(name="name")
        )

    def test_list_column_specs(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [],
            },
            {},
        )
        client.list_column_specs(dataset_name="name")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )

    def test_update_column_spec_not_found(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(type_code=automl_v1beta1.TypeCode.STRING),
        )

        client = self.tables_client(
            client_attrs={
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            prediction_client_attrs={},
        )
        with pytest.raises(exceptions.NotFound):
            client.update_column_spec(dataset_name="name", column_spec_name="column2")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_not_called()

    def test_update_column_spec_display_name_not_found(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(type_code=automl_v1beta1.TypeCode.STRING),
        )
        client = self.tables_client(
            client_attrs={
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            prediction_client_attrs={},
        )
        with pytest.raises(exceptions.NotFound):
            client.update_column_spec(
                dataset_name="name", column_spec_display_name="column2"
            )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_not_called()

    def test_update_column_spec_name_no_args(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec = automl_v1beta1.ColumnSpec(
            name="column/2",
            display_name="column",
            data_type=automl_v1beta1.DataType(
                type_code=automl_v1beta1.TypeCode.FLOAT64
            ),
        )

        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            {},
        )
        client.update_column_spec(dataset_name="name", column_spec_name="column/2")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_called_with(
            request=automl_v1beta1.UpdateColumnSpecRequest(
                column_spec={
                    "name": "column/2",
                    "data_type": {"type_code": automl_v1beta1.TypeCode.FLOAT64},
                }
            )
        )

    def test_update_column_spec_no_args(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(
                type_code=automl_v1beta1.TypeCode.FLOAT64
            ),
        )

        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            {},
        )
        client.update_column_spec(
            dataset_name="name", column_spec_display_name="column"
        )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_called_with(
            request=automl_v1beta1.UpdateColumnSpecRequest(
                column_spec={
                    "name": "column",
                    "data_type": {"type_code": automl_v1beta1.TypeCode.FLOAT64},
                }
            )
        )

    def test_update_column_spec_nullable(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(
                type_code=automl_v1beta1.TypeCode.FLOAT64
            ),
        )

        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            {},
        )
        client.update_column_spec(
            dataset_name="name", column_spec_display_name="column", nullable=True
        )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_called_with(
            request=automl_v1beta1.UpdateColumnSpecRequest(
                column_spec={
                    "name": "column",
                    "data_type": {
                        "type_code": automl_v1beta1.TypeCode.FLOAT64,
                        "nullable": True,
                    },
                }
            )
        )

    def test_update_column_spec_type_code(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(
                type_code=automl_v1beta1.TypeCode.FLOAT64
            ),
        )
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            {},
        )
        client.update_column_spec(
            dataset_name="name",
            column_spec_display_name="column",
            type_code=automl_v1beta1.TypeCode.ARRAY,
        )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_called_with(
            request=automl_v1beta1.UpdateColumnSpecRequest(
                column_spec={
                    "name": "column",
                    "data_type": {"type_code": automl_v1beta1.TypeCode.ARRAY},
                }
            )
        )

    def test_update_column_spec_type_code_nullable(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(
                type_code=automl_v1beta1.TypeCode.FLOAT64
            ),
        )
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            {},
        )
        client.update_column_spec(
            dataset_name="name",
            nullable=True,
            column_spec_display_name="column",
            type_code=automl_v1beta1.TypeCode.ARRAY,
        )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_called_with(
            request=automl_v1beta1.UpdateColumnSpecRequest(
                column_spec={
                    "name": "column",
                    "data_type": {
                        "type_code": automl_v1beta1.TypeCode.ARRAY,
                        "nullable": True,
                    },
                }
            )
        )

    def test_update_column_spec_type_code_nullable_false(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec = automl_v1beta1.ColumnSpec(
            name="column",
            display_name="column",
            data_type=automl_v1beta1.DataType(
                type_code=automl_v1beta1.TypeCode.FLOAT64
            ),
        )
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec],
            },
            {},
        )
        client.update_column_spec(
            dataset_name="name",
            nullable=False,
            column_spec_display_name="column",
            type_code=automl_v1beta1.TypeCode.FLOAT64,
        )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_column_spec.assert_called_with(
            request=automl_v1beta1.UpdateColumnSpecRequest(
                column_spec={
                    "name": "column",
                    "data_type": {
                        "type_code": automl_v1beta1.TypeCode.FLOAT64,
                        "nullable": False,
                    },
                }
            )
        )

    def test_set_target_column_table_not_found(self):
        client = self.tables_client(
            {"list_table_specs.side_effect": exceptions.NotFound("err")}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.set_target_column(
                dataset_name="name", column_spec_display_name="column2"
            )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_not_called()
        client.auto_ml_client.update_dataset.assert_not_called()

    def test_set_target_column_not_found(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/1", display_name="column")
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        with pytest.raises(exceptions.NotFound):
            client.set_target_column(
                dataset_name="name", column_spec_display_name="column2"
            )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_dataset.assert_not_called()

    def test_set_target_column(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/1", display_name="column")
        dataset_mock = mock.Mock()
        tables_dataset_metadata_mock = mock.Mock()
        tables_dataset_metadata_mock.configure_mock(
            target_column_spec_id="2",
            weight_column_spec_id="2",
            ml_use_column_spec_id="3",
        )
        dataset_mock.configure_mock(
            name="dataset", tables_dataset_metadata=tables_dataset_metadata_mock
        )
        client = self.tables_client(
            {
                "get_dataset.return_value": dataset_mock,
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        client.set_target_column(dataset_name="name", column_spec_display_name="column")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_dataset.assert_called_with(
            request=automl_v1beta1.UpdateDatasetRequest(
                dataset={
                    "name": "dataset",
                    "tables_dataset_metadata": {
                        "target_column_spec_id": "1",
                        "weight_column_spec_id": "2",
                        "ml_use_column_spec_id": "3",
                    },
                }
            )
        )

    def test_set_weight_column_table_not_found(self):
        client = self.tables_client(
            {"list_table_specs.side_effect": exceptions.NotFound("err")}, {}
        )
        try:
            client.set_weight_column(
                dataset_name="name", column_spec_display_name="column2"
            )
        except exceptions.NotFound:
            pass
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_not_called()
        client.auto_ml_client.update_dataset.assert_not_called()

    def test_set_weight_column_not_found(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/1", display_name="column")
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        with pytest.raises(exceptions.NotFound):
            client.set_weight_column(
                dataset_name="name", column_spec_display_name="column2"
            )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_dataset.assert_not_called()

    def test_set_weight_column(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/2", display_name="column")
        dataset_mock = mock.Mock()
        tables_dataset_metadata_mock = mock.Mock()
        tables_dataset_metadata_mock.configure_mock(
            target_column_spec_id="1",
            weight_column_spec_id="1",
            ml_use_column_spec_id="3",
        )
        dataset_mock.configure_mock(
            name="dataset", tables_dataset_metadata=tables_dataset_metadata_mock
        )
        client = self.tables_client(
            {
                "get_dataset.return_value": dataset_mock,
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        client.set_weight_column(dataset_name="name", column_spec_display_name="column")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_dataset.assert_called_with(
            request=automl_v1beta1.UpdateDatasetRequest(
                dataset={
                    "name": "dataset",
                    "tables_dataset_metadata": {
                        "target_column_spec_id": "1",
                        "weight_column_spec_id": "2",
                        "ml_use_column_spec_id": "3",
                    },
                }
            )
        )

    def test_clear_weight_column(self):
        dataset_mock = mock.Mock()
        tables_dataset_metadata_mock = mock.Mock()
        tables_dataset_metadata_mock.configure_mock(
            target_column_spec_id="1",
            weight_column_spec_id="2",
            ml_use_column_spec_id="3",
        )
        dataset_mock.configure_mock(
            name="dataset", tables_dataset_metadata=tables_dataset_metadata_mock
        )
        client = self.tables_client({"get_dataset.return_value": dataset_mock}, {})
        client.clear_weight_column(dataset_name="name")
        client.auto_ml_client.update_dataset.assert_called_with(
            request=automl_v1beta1.UpdateDatasetRequest(
                dataset={
                    "name": "dataset",
                    "tables_dataset_metadata": {
                        "target_column_spec_id": "1",
                        "weight_column_spec_id": None,
                        "ml_use_column_spec_id": "3",
                    },
                }
            )
        )

    def test_set_test_train_column_table_not_found(self):
        client = self.tables_client(
            {"list_table_specs.side_effect": exceptions.NotFound("err")}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.set_test_train_column(
                dataset_name="name", column_spec_display_name="column2"
            )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_not_called()
        client.auto_ml_client.update_dataset.assert_not_called()

    def test_set_test_train_column_not_found(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/1", display_name="column")
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        with pytest.raises(exceptions.NotFound):
            client.set_test_train_column(
                dataset_name="name", column_spec_display_name="column2"
            )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_dataset.assert_not_called()

    def test_set_test_train_column(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/3", display_name="column")
        dataset_mock = mock.Mock()
        tables_dataset_metadata_mock = mock.Mock()
        tables_dataset_metadata_mock.configure_mock(
            target_column_spec_id="1",
            weight_column_spec_id="2",
            ml_use_column_spec_id="2",
        )
        dataset_mock.configure_mock(
            name="dataset", tables_dataset_metadata=tables_dataset_metadata_mock
        )
        client = self.tables_client(
            {
                "get_dataset.return_value": dataset_mock,
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        client.set_test_train_column(
            dataset_name="name", column_spec_display_name="column"
        )
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_dataset.assert_called_with(
            request=automl_v1beta1.UpdateDatasetRequest(
                dataset={
                    "name": "dataset",
                    "tables_dataset_metadata": {
                        "target_column_spec_id": "1",
                        "weight_column_spec_id": "2",
                        "ml_use_column_spec_id": "3",
                    },
                }
            )
        )

    def test_clear_test_train_column(self):
        dataset_mock = mock.Mock()
        tables_dataset_metadata_mock = mock.Mock()
        tables_dataset_metadata_mock.configure_mock(
            target_column_spec_id="1",
            weight_column_spec_id="2",
            ml_use_column_spec_id="2",
        )
        dataset_mock.configure_mock(
            name="dataset", tables_dataset_metadata=tables_dataset_metadata_mock
        )
        client = self.tables_client({"get_dataset.return_value": dataset_mock}, {})
        client.clear_test_train_column(dataset_name="name")
        client.auto_ml_client.update_dataset.assert_called_with(
            request=automl_v1beta1.UpdateDatasetRequest(
                dataset={
                    "name": "dataset",
                    "tables_dataset_metadata": {
                        "target_column_spec_id": "1",
                        "weight_column_spec_id": "2",
                        "ml_use_column_spec_id": None,
                    },
                }
            )
        )

    def test_set_time_column(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/3", display_name="column")
        dataset_mock = mock.Mock()
        dataset_mock.configure_mock(name="dataset")
        client = self.tables_client(
            {
                "get_dataset.return_value": dataset_mock,
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
            },
            {},
        )
        client.set_time_column(dataset_name="name", column_spec_display_name="column")
        client.auto_ml_client.list_table_specs.assert_called_with(
            request=automl_v1beta1.ListTableSpecsRequest(parent="name")
        )
        client.auto_ml_client.list_column_specs.assert_called_with(
            request=automl_v1beta1.ListColumnSpecsRequest(parent="table")
        )
        client.auto_ml_client.update_table_spec.assert_called_with(
            request=automl_v1beta1.UpdateTableSpecRequest(
                table_spec={"name": "table", "time_column_spec_id": "3"}
            )
        )

    def test_clear_time_column(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        dataset_mock = mock.Mock()
        dataset_mock.configure_mock(name="dataset")
        client = self.tables_client(
            {
                "get_dataset.return_value": dataset_mock,
                "list_table_specs.return_value": [table_spec_mock],
            },
            {},
        )
        client.clear_time_column(dataset_name="name")
        client.auto_ml_client.update_table_spec.assert_called_with(
            request=automl_v1beta1.UpdateTableSpecRequest(
                table_spec={"name": "table", "time_column_spec_id": None}
            )
        )

    def test_get_model_evaluation(self):
        client = self.tables_client({}, {})
        client.get_model_evaluation(model_evaluation_name="x")
        client.auto_ml_client.get_model_evaluation.assert_called_with(
            request=automl_v1beta1.GetModelEvaluationRequest(name="x")
        )

    def test_list_model_evaluations_empty(self):
        client = self.tables_client({"list_model_evaluations.return_value": []}, {})
        ds = client.list_model_evaluations(model_name="model")
        client.auto_ml_client.list_model_evaluations.assert_called_with(
            request=automl_v1beta1.ListModelEvaluationsRequest(parent="model")
        )
        assert ds == []

    def test_list_model_evaluations_not_empty(self):
        evaluations = ["eval"]
        client = self.tables_client(
            {
                "list_model_evaluations.return_value": evaluations,
                "location_path.return_value": LOCATION_PATH,
            },
            {},
        )
        ds = client.list_model_evaluations(model_name="model")
        client.auto_ml_client.list_model_evaluations.assert_called_with(
            request=automl_v1beta1.ListModelEvaluationsRequest(parent="model")
        )
        assert len(ds) == 1
        assert ds[0] == "eval"

    def test_list_models_empty(self):
        client = self.tables_client(
            {
                "list_models.return_value": [],
                "location_path.return_value": LOCATION_PATH,
            },
            {},
        )
        ds = client.list_models()

        client.auto_ml_client.list_models.assert_called_with(
            request=automl_v1beta1.ListModelsRequest(parent=LOCATION_PATH)
        )
        assert ds == []

    def test_list_models_not_empty(self):
        models = ["some_model"]
        client = self.tables_client(
            {
                "list_models.return_value": models,
                "location_path.return_value": LOCATION_PATH,
            },
            {},
        )
        ds = client.list_models()

        client.auto_ml_client.list_models.assert_called_with(
            request=automl_v1beta1.ListModelsRequest(parent=LOCATION_PATH)
        )
        assert len(ds) == 1
        assert ds[0] == "some_model"

    def test_get_model_name(self):
        model_actual = "model"
        client = self.tables_client({"get_model.return_value": model_actual}, {})
        model = client.get_model(model_name="my_model")
        client.auto_ml_client.get_model.assert_called_with(name="my_model")
        assert model == model_actual

    def test_get_no_model(self):
        client = self.tables_client(
            {"get_model.side_effect": exceptions.NotFound("err")}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.get_model(model_name="my_model")
        client.auto_ml_client.get_model.assert_called_with(name="my_model")

    def test_get_model_from_empty_list(self):
        client = self.tables_client({"list_models.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.get_model(model_display_name="my_model")

    def test_get_model_from_list_not_found(self):
        client = self.tables_client(
            {"list_models.return_value": [mock.Mock(display_name="not_it")]}, {}
        )
        with pytest.raises(exceptions.NotFound):
            client.get_model(model_display_name="my_model")

    def test_get_model_from_list(self):
        client = self.tables_client(
            {
                "list_models.return_value": [
                    mock.Mock(display_name="not_it"),
                    mock.Mock(display_name="my_model"),
                ]
            },
            {},
        )
        model = client.get_model(model_display_name="my_model")
        assert model.display_name == "my_model"

    def test_get_model_from_list_ambiguous(self):
        client = self.tables_client(
            {
                "list_models.return_value": [
                    mock.Mock(display_name="my_model"),
                    mock.Mock(display_name="not_my_model"),
                    mock.Mock(display_name="my_model"),
                ]
            },
            {},
        )
        with pytest.raises(ValueError):
            client.get_model(model_display_name="my_model")

    def test_delete_model(self):
        model = mock.Mock()
        model.configure_mock(name="name")
        client = self.tables_client({"delete_model.return_value": None}, {})
        client.delete_model(model=model)
        client.auto_ml_client.delete_model.assert_called_with(
            request=automl_v1beta1.DeleteModelRequest(name="name")
        )

    def test_delete_model_not_found(self):
        client = self.tables_client({"list_models.return_value": []}, {})
        client.delete_model(model_display_name="not_found")
        client.auto_ml_client.delete_model.assert_not_called()

    def test_delete_model_name(self):
        client = self.tables_client({"delete_model.return_value": None}, {})
        client.delete_model(model_name="name")
        client.auto_ml_client.delete_model.assert_called_with(
            request=automl_v1beta1.DeleteModelRequest(name="name")
        )

    def test_deploy_model_no_args(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.deploy_model()
        client.auto_ml_client.deploy_model.assert_not_called()

    def test_deploy_model(self):
        client = self.tables_client({}, {})
        client.deploy_model(model_name="name")
        client.auto_ml_client.deploy_model.assert_called_with(
            request=automl_v1beta1.DeployModelRequest(name="name")
        )

    def test_deploy_model_not_found(self):
        client = self.tables_client({"list_models.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.deploy_model(model_display_name="name")
        client.auto_ml_client.deploy_model.assert_not_called()

    def test_undeploy_model(self):
        client = self.tables_client({}, {})
        client.undeploy_model(model_name="name")
        client.auto_ml_client.undeploy_model.assert_called_with(
            request=automl_v1beta1.UndeployModelRequest(name="name")
        )

    def test_undeploy_model_not_found(self):
        client = self.tables_client({"list_models.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.undeploy_model(model_display_name="name")
        client.auto_ml_client.undeploy_model.assert_not_called()

    def test_create_model(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")
        column_spec_mock = mock.Mock()
        column_spec_mock.configure_mock(name="column/2", display_name="column")
        client = self.tables_client(
            {
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_mock],
                "location_path.return_value": LOCATION_PATH,
            },
            {},
        )
        client.create_model(
            "my_model", dataset_name="my_dataset", train_budget_milli_node_hours=1000
        )
        client.auto_ml_client.create_model.assert_called_with(
            request=automl_v1beta1.CreateModelRequest(
                parent=LOCATION_PATH,
                model={
                    "display_name": "my_model",
                    "dataset_id": "my_dataset",
                    "tables_model_metadata": {"train_budget_milli_node_hours": 1000},
                },
            )
        )

    def test_create_model_include_columns(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec_1 = automl_v1beta1.ColumnSpec(
            name="column/1", display_name="column1"
        )
        column_spec_2 = automl_v1beta1.ColumnSpec(
            name="column/2", display_name="column2"
        )

        client = self.tables_client(
            client_attrs={
                "list_table_specs.return_value": [
                    automl_v1beta1.TableSpec(name="table")
                ],
                "list_column_specs.return_value": [column_spec_1, column_spec_2],
                "location_path.return_value": LOCATION_PATH,
            },
            prediction_client_attrs={},
        )
        client.create_model(
            "my_model",
            dataset_name="my_dataset",
            include_column_spec_names=["column1"],
            train_budget_milli_node_hours=1000,
        )
        client.auto_ml_client.create_model.assert_called_with(
            request=automl_v1beta1.CreateModelRequest(
                parent=LOCATION_PATH,
                model=automl_v1beta1.Model(
                    display_name="my_model",
                    dataset_id="my_dataset",
                    tables_model_metadata=automl_v1beta1.TablesModelMetadata(
                        train_budget_milli_node_hours=1000,
                        input_feature_column_specs=[column_spec_1],
                    ),
                ),
            )
        )

    def test_create_model_exclude_columns(self):
        table_spec_mock = mock.Mock()
        # name is reserved in use of __init__, needs to be passed here
        table_spec_mock.configure_mock(name="table")

        column_spec_1 = automl_v1beta1.ColumnSpec(
            name="column/1", display_name="column1"
        )
        column_spec_2 = automl_v1beta1.ColumnSpec(
            name="column/2", display_name="column2"
        )
        client = self.tables_client(
            client_attrs={
                "list_table_specs.return_value": [table_spec_mock],
                "list_column_specs.return_value": [column_spec_1, column_spec_2],
                "location_path.return_value": LOCATION_PATH,
            },
            prediction_client_attrs={},
        )
        client.create_model(
            "my_model",
            dataset_name="my_dataset",
            exclude_column_spec_names=["column1"],
            train_budget_milli_node_hours=1000,
        )
        client.auto_ml_client.create_model.assert_called_with(
            request=automl_v1beta1.CreateModelRequest(
                parent=LOCATION_PATH,
                model=automl_v1beta1.Model(
                    display_name="my_model",
                    dataset_id="my_dataset",
                    tables_model_metadata=automl_v1beta1.TablesModelMetadata(
                        train_budget_milli_node_hours=1000,
                        input_feature_column_specs=[column_spec_2],
                    ),
                ),
            )
        )

    def test_create_model_invalid_hours_small(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.create_model(
                "my_model", dataset_name="my_dataset", train_budget_milli_node_hours=1
            )
        client.auto_ml_client.create_model.assert_not_called()

    def test_create_model_invalid_hours_large(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.create_model(
                "my_model",
                dataset_name="my_dataset",
                train_budget_milli_node_hours=1000000,
            )
        client.auto_ml_client.create_model.assert_not_called()

    def test_create_model_invalid_no_dataset(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.create_model("my_model", train_budget_milli_node_hours=1000)
        client.auto_ml_client.get_dataset.assert_not_called()
        client.auto_ml_client.create_model.assert_not_called()

    def test_create_model_invalid_include_exclude(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.create_model(
                "my_model",
                dataset_name="my_dataset",
                include_column_spec_names=["a"],
                exclude_column_spec_names=["b"],
                train_budget_milli_node_hours=1000,
            )
        client.auto_ml_client.get_dataset.assert_not_called()
        client.auto_ml_client.create_model.assert_not_called()

    def test_predict_from_array(self):
        data_type = mock.Mock(type_code=data_types.TypeCode.CATEGORY)
        column_spec = mock.Mock(display_name="a", data_type=data_type)
        model_metadata = mock.Mock(input_feature_column_specs=[column_spec])
        model = mock.Mock()
        model.configure_mock(tables_model_metadata=model_metadata, name="my_model")
        client = self.tables_client({"get_model.return_value": model}, {})
        client.predict(["1"], model_name="my_model")

        # append each row value separately until issue is resovled
        # https://github.com/googleapis/proto-plus-python/issues/104
        row = data_items.Row()
        row.values.append(struct.Value(string_value="1"))
        payload = data_items.ExamplePayload(row=row)

        client.prediction_client.predict.assert_called_with(
            request=automl_v1beta1.PredictRequest(
                name="my_model", payload=payload, params=None
            )
        )

    def test_predict_from_dict(self):
        data_type = mock.Mock(type_code=data_types.TypeCode.CATEGORY)
        column_spec_a = mock.Mock(display_name="a", data_type=data_type)
        column_spec_b = mock.Mock(display_name="b", data_type=data_type)
        model_metadata = mock.Mock(
            input_feature_column_specs=[column_spec_a, column_spec_b]
        )
        model = mock.Mock()
        model.configure_mock(tables_model_metadata=model_metadata, name="my_model")
        client = self.tables_client({"get_model.return_value": model}, {})
        client.predict({"a": "1", "b": "2"}, model_name="my_model")

        # append each row value separately until issue is resovled
        # https://github.com/googleapis/proto-plus-python/issues/104
        row = data_items.Row()
        row.values.append(struct.Value(string_value="1"))
        row.values.append(struct.Value(string_value="2"))

        payload = data_items.ExamplePayload(row=row)

        client.prediction_client.predict.assert_called_with(
            request=automl_v1beta1.PredictRequest(
                name="my_model", payload=payload, params=None
            )
        )

    def test_predict_from_dict_with_feature_importance(self):
        data_type = mock.Mock(type_code=data_types.TypeCode.CATEGORY)
        column_spec_a = mock.Mock(display_name="a", data_type=data_type)
        column_spec_b = mock.Mock(display_name="b", data_type=data_type)
        model_metadata = mock.Mock(
            input_feature_column_specs=[column_spec_a, column_spec_b]
        )
        model = mock.Mock()
        model.configure_mock(tables_model_metadata=model_metadata, name="my_model")
        client = self.tables_client({"get_model.return_value": model}, {})
        client.predict(
            {"a": "1", "b": "2"}, model_name="my_model", feature_importance=True
        )

        # append each row value separately until issue is resovled
        # https://github.com/googleapis/proto-plus-python/issues/104
        row = data_items.Row()
        row.values.append(struct.Value(string_value="1"))
        row.values.append(struct.Value(string_value="2"))

        payload = data_items.ExamplePayload(row=row)

        client.prediction_client.predict.assert_called_with(
            request=automl_v1beta1.PredictRequest(
                name="my_model", payload=payload, params={"feature_importance": "true"}
            )
        )

    def test_predict_from_dict_missing(self):
        data_type = mock.Mock(type_code=data_types.TypeCode.CATEGORY)
        column_spec_a = mock.Mock(display_name="a", data_type=data_type)
        column_spec_b = mock.Mock(display_name="b", data_type=data_type)
        model_metadata = mock.Mock(
            input_feature_column_specs=[column_spec_a, column_spec_b]
        )
        model = mock.Mock()
        model.configure_mock(tables_model_metadata=model_metadata, name="my_model")
        client = self.tables_client({"get_model.return_value": model}, {})
        client.predict({"a": "1"}, model_name="my_model")

        # append each row value separately until issue is resovled
        # https://github.com/googleapis/proto-plus-python/issues/104
        row = data_items.Row()
        row.values.append(struct.Value(string_value="1"))
        row.values.append(struct.Value(null_value=struct.NullValue.NULL_VALUE))

        payload = data_items.ExamplePayload(row=row)

        client.prediction_client.predict.assert_called_with(
            request=automl_v1beta1.PredictRequest(
                name="my_model", payload=payload, params=None
            )
        )

    def test_predict_all_types(self):
        float_type = mock.Mock(type_code=data_types.TypeCode.FLOAT64)
        timestamp_type = mock.Mock(type_code=data_types.TypeCode.TIMESTAMP)
        string_type = mock.Mock(type_code=data_types.TypeCode.STRING)
        array_type = mock.Mock(
            type_code=data_types.TypeCode.ARRAY,
            list_element_type=mock.Mock(type_code=data_types.TypeCode.FLOAT64),
        )

        struct_type = mock.Mock(
            type_code=data_types.TypeCode.STRUCT,
            struct_type=data_types.StructType(
                fields={
                    "a": data_types.DataType(type_code=data_types.TypeCode.CATEGORY),
                    "b": data_types.DataType(type_code=data_types.TypeCode.CATEGORY),
                }
            ),
        )
        category_type = mock.Mock(type_code=data_types.TypeCode.CATEGORY)
        column_spec_float = mock.Mock(display_name="float", data_type=float_type)
        column_spec_timestamp = mock.Mock(
            display_name="timestamp", data_type=timestamp_type
        )
        column_spec_string = mock.Mock(display_name="string", data_type=string_type)
        column_spec_array = mock.Mock(display_name="array", data_type=array_type)
        column_spec_struct = mock.Mock(display_name="struct", data_type=struct_type)
        column_spec_category = mock.Mock(
            display_name="category", data_type=category_type
        )
        column_spec_null = mock.Mock(display_name="null", data_type=category_type)
        model_metadata = mock.Mock(
            input_feature_column_specs=[
                column_spec_float,
                column_spec_timestamp,
                column_spec_string,
                column_spec_array,
                column_spec_struct,
                column_spec_category,
                column_spec_null,
            ]
        )
        model = mock.Mock()
        model.configure_mock(tables_model_metadata=model_metadata, name="my_model")
        client = self.tables_client({"get_model.return_value": model}, {})
        client.predict(
            {
                "float": 1.0,
                "timestamp": "EST",
                "string": "text",
                "array": [1],
                "struct": {"a": "label_a", "b": "label_b"},
                "category": "a",
                "null": None,
            },
            model_name="my_model",
        )
        struct_pb = struct.Struct()
        struct_pb.fields["a"].CopyFrom(struct.Value(string_value="label_a"))
        struct_pb.fields["b"].CopyFrom(struct.Value(string_value="label_b"))

        # append each row value separately until issue is resovled
        # https://github.com/googleapis/proto-plus-python/issues/104
        row = data_items.Row()
        values = [
            struct.Value(number_value=1.0),
            struct.Value(string_value="EST"),
            struct.Value(string_value="text"),
            struct.Value(
                list_value=struct.ListValue(values=[struct.Value(number_value=1.0)])
            ),
            struct.Value(struct_value=struct_pb),
            struct.Value(string_value="a"),
            struct.Value(null_value=struct.NullValue.NULL_VALUE),
        ]
        for v in values:
            row.values.append(v)

        payload = data_items.ExamplePayload(row=row)

        client.prediction_client.predict.assert_called_with(
            request=automl_v1beta1.PredictRequest(
                name="my_model", payload=payload, params=None
            )
        )

    def test_predict_from_array_missing(self):
        data_type = mock.Mock(type_code=data_types.TypeCode.CATEGORY)
        column_spec = mock.Mock(display_name="a", data_type=data_type)
        model_metadata = mock.Mock(input_feature_column_specs=[column_spec])
        model = mock.Mock()
        model.configure_mock(tables_model_metadata=model_metadata, name="my_model")
        client = self.tables_client({"get_model.return_value": model}, {})
        with pytest.raises(ValueError):
            client.predict([], model_name="my_model")
        client.prediction_client.predict.assert_not_called()

    def test_batch_predict_pandas_dataframe(self):
        client = self.tables_client(
            gcs_client_attrs={
                "bucket_name": "my_bucket",
                "upload_pandas_dataframe.return_value": "gs://input",
            }
        )
        dataframe = pandas.DataFrame({})
        client.batch_predict(
            project=PROJECT,
            region=REGION,
            model_name="my_model",
            pandas_dataframe=dataframe,
            gcs_output_uri_prefix="gs://output",
        )

        client.gcs_client.ensure_bucket_exists.assert_called_with(PROJECT, REGION)
        client.gcs_client.upload_pandas_dataframe.assert_called_with(dataframe)

        client.prediction_client.batch_predict.assert_called_with(
            request=automl_v1beta1.BatchPredictRequest(
                name="my_model",
                input_config={"gcs_source": {"input_uris": ["gs://input"]}},
                output_config={"gcs_destination": {"output_uri_prefix": "gs://output"}},
            )
        )

    def test_batch_predict_pandas_dataframe_init_gcs(self):
        client = automl_v1beta1.TablesClient(
            client=mock.Mock(),
            prediction_client=mock.Mock(),
            project=PROJECT,
            region=REGION,
            credentials=AnonymousCredentials(),
        )

        dataframe = pandas.DataFrame({})
        patch = mock.patch(
            "google.cloud.automl_v1beta1.services.tables.gcs_client.GcsClient",
            bucket_name="my_bucket",
        )
        with patch as MockGcsClient:
            mockInstance = MockGcsClient.return_value
            mockInstance.upload_pandas_dataframe.return_value = "gs://input"

            dataframe = pandas.DataFrame({})
            client.batch_predict(
                model_name="my_model",
                pandas_dataframe=dataframe,
                gcs_output_uri_prefix="gs://output",
            )

            client.gcs_client.ensure_bucket_exists.assert_called_with(PROJECT, REGION)
            client.gcs_client.upload_pandas_dataframe.assert_called_with(dataframe)

            client.prediction_client.batch_predict.assert_called_with(
                request=automl_v1beta1.BatchPredictRequest(
                    name="my_model",
                    input_config={"gcs_source": {"input_uris": ["gs://input"]}},
                    output_config={
                        "gcs_destination": {"output_uri_prefix": "gs://output"}
                    },
                )
            )

    def test_batch_predict_gcs(self):
        client = self.tables_client({}, {})
        client.batch_predict(
            model_name="my_model",
            gcs_input_uris="gs://input",
            gcs_output_uri_prefix="gs://output",
        )
        client.prediction_client.batch_predict.assert_called_with(
            request=automl_v1beta1.BatchPredictRequest(
                name="my_model",
                input_config={"gcs_source": {"input_uris": ["gs://input"]}},
                output_config={"gcs_destination": {"output_uri_prefix": "gs://output"}},
            )
        )

    def test_batch_predict_bigquery(self):
        client = self.tables_client({}, {})
        client.batch_predict(
            model_name="my_model",
            bigquery_input_uri="bq://input",
            bigquery_output_uri="bq://output",
        )
        client.prediction_client.batch_predict.assert_called_with(
            request=automl_v1beta1.BatchPredictRequest(
                name="my_model",
                input_config={"bigquery_source": {"input_uri": "bq://input"}},
                output_config={"bigquery_destination": {"output_uri": "bq://output"}},
            )
        )

    def test_batch_predict_bigquery_with_params(self):
        client = self.tables_client({}, {})
        client.batch_predict(
            model_name="my_model",
            bigquery_input_uri="bq://input",
            bigquery_output_uri="bq://output",
            params={"feature_importance": "true"},
        )

        client.prediction_client.batch_predict.assert_called_with(
            request=automl_v1beta1.BatchPredictRequest(
                name="my_model",
                input_config={"bigquery_source": {"input_uri": "bq://input"}},
                output_config={"bigquery_destination": {"output_uri": "bq://output"}},
                params={"feature_importance": "true"},
            )
        )

    def test_batch_predict_mixed(self):
        client = self.tables_client({}, {})
        client.batch_predict(
            model_name="my_model",
            gcs_input_uris="gs://input",
            bigquery_output_uri="bq://output",
        )
        client.prediction_client.batch_predict.assert_called_with(
            request=automl_v1beta1.BatchPredictRequest(
                name="my_model",
                input_config={"gcs_source": {"input_uris": ["gs://input"]}},
                output_config={"bigquery_destination": {"output_uri": "bq://output"}},
            )
        )

    def test_batch_predict_missing_input_gcs_uri(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.batch_predict(
                model_name="my_model",
                gcs_input_uris=None,
                gcs_output_uri_prefix="gs://output",
            )
        client.prediction_client.batch_predict.assert_not_called()

    def test_batch_predict_missing_input_bigquery_uri(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.batch_predict(
                model_name="my_model",
                bigquery_input_uri=None,
                gcs_output_uri_prefix="gs://output",
            )
        client.prediction_client.batch_predict.assert_not_called()

    def test_batch_predict_missing_output_gcs_uri(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.batch_predict(
                model_name="my_model",
                gcs_input_uris="gs://input",
                gcs_output_uri_prefix=None,
            )
        client.prediction_client.batch_predict.assert_not_called()

    def test_batch_predict_missing_output_bigquery_uri(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.batch_predict(
                model_name="my_model",
                gcs_input_uris="gs://input",
                bigquery_output_uri=None,
            )
        client.prediction_client.batch_predict.assert_not_called()

    def test_batch_predict_missing_model(self):
        client = self.tables_client({"list_models.return_value": []}, {})
        with pytest.raises(exceptions.NotFound):
            client.batch_predict(
                model_display_name="my_model",
                gcs_input_uris="gs://input",
                gcs_output_uri_prefix="gs://output",
            )
        client.prediction_client.batch_predict.assert_not_called()

    def test_batch_predict_no_model(self):
        client = self.tables_client({}, {})
        with pytest.raises(ValueError):
            client.batch_predict(
                gcs_input_uris="gs://input", gcs_output_uri_prefix="gs://output"
            )
        client.auto_ml_client.list_models.assert_not_called()
        client.prediction_client.batch_predict.assert_not_called()

    def test_auto_ml_client_credentials(self):
        credentials_mock = mock.Mock()
        patch_auto_ml_client = mock.patch(
            "google.cloud.automl_v1beta1.services.tables.tables_client.AutoMlClient"
        )
        with patch_auto_ml_client as MockAutoMlClient:
            automl_v1beta1.TablesClient(credentials=credentials_mock)
        _, auto_ml_client_kwargs = MockAutoMlClient.call_args
        assert "credentials" in auto_ml_client_kwargs
        assert auto_ml_client_kwargs["credentials"] == credentials_mock

    def test_prediction_client_credentials(self):
        credentials_mock = mock.Mock()
        patch_prediction_client = mock.patch(
            "google.cloud.automl_v1beta1.services.tables.tables_client.PredictionServiceClient"
        )
        with patch_prediction_client as MockPredictionClient:
            automl_v1beta1.TablesClient(credentials=credentials_mock)
        _, prediction_client_kwargs = MockPredictionClient.call_args
        assert "credentials" in prediction_client_kwargs
        assert prediction_client_kwargs["credentials"] == credentials_mock

    def test_prediction_client_client_info(self):
        credentials_mock = mock.Mock()
        client_info_mock = mock.Mock()
        patch_prediction_client = mock.patch(
            "google.cloud.automl_v1beta1.services.tables.tables_client.PredictionServiceClient"
        )
        with patch_prediction_client as MockPredictionClient:
            automl_v1beta1.TablesClient(
                credentials=credentials_mock,
                client_info=client_info_mock,
            )
        _, prediction_client_kwargs = MockPredictionClient.call_args
        assert "client_info" in prediction_client_kwargs
        assert prediction_client_kwargs["client_info"] == client_info_mock



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.automl_v1beta1.types import io

__protobuf__ = proto.module(
    package="google.cloud.automl.v1beta1",
    manifest={
        "OperationMetadata",
        "DeleteOperationMetadata",
        "DeployModelOperationMetadata",
        "UndeployModelOperationMetadata",
        "CreateModelOperationMetadata",
        "ImportDataOperationMetadata",
        "ExportDataOperationMetadata",
        "BatchPredictOperationMetadata",
        "ExportModelOperationMetadata",
        "ExportEvaluatedExamplesOperationMetadata",
    },
)


class OperationMetadata(proto.Message):
    r"""Metadata used across all long running operations returned by
    AutoML API.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        delete_details (google.cloud.automl_v1beta1.types.DeleteOperationMetadata):
            Details of a Delete operation.

            This field is a member of `oneof`_ ``details``.
        deploy_model_details (google.cloud.automl_v1beta1.types.DeployModelOperationMetadata):
            Details of a DeployModel operation.

            This field is a member of `oneof`_ ``details``.
        undeploy_model_details (google.cloud.automl_v1beta1.types.UndeployModelOperationMetadata):
            Details of an UndeployModel operation.

            This field is a member of `oneof`_ ``details``.
        create_model_details (google.cloud.automl_v1beta1.types.CreateModelOperationMetadata):
            Details of CreateModel operation.

            This field is a member of `oneof`_ ``details``.
        import_data_details (google.cloud.automl_v1beta1.types.ImportDataOperationMetadata):
            Details of ImportData operation.

            This field is a member of `oneof`_ ``details``.
        batch_predict_details (google.cloud.automl_v1beta1.types.BatchPredictOperationMetadata):
            Details of BatchPredict operation.

            This field is a member of `oneof`_ ``details``.
        export_data_details (google.cloud.automl_v1beta1.types.ExportDataOperationMetadata):
            Details of ExportData operation.

            This field is a member of `oneof`_ ``details``.
        export_model_details (google.cloud.automl_v1beta1.types.ExportModelOperationMetadata):
            Details of ExportModel operation.

            This field is a member of `oneof`_ ``details``.
        export_evaluated_examples_details (google.cloud.automl_v1beta1.types.ExportEvaluatedExamplesOperationMetadata):
            Details of ExportEvaluatedExamples operation.

            This field is a member of `oneof`_ ``details``.
        progress_percent (int):
            Output only. Progress of operation. Range: [0, 100]. Not
            used currently.
        partial_failures (MutableSequence[google.rpc.status_pb2.Status]):
            Output only. Partial failures encountered.
            E.g. single files that couldn't be read.
            This field should never exceed 20 entries.
            Status details field will contain standard GCP
            error details.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the operation was
            created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the operation was
            updated for the last time.
    """

    delete_details: "DeleteOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="details",
        message="DeleteOperationMetadata",
    )
    deploy_model_details: "DeployModelOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=24,
        oneof="details",
        message="DeployModelOperationMetadata",
    )
    undeploy_model_details: "UndeployModelOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=25,
        oneof="details",
        message="UndeployModelOperationMetadata",
    )
    create_model_details: "CreateModelOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="details",
        message="CreateModelOperationMetadata",
    )
    import_data_details: "ImportDataOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="details",
        message="ImportDataOperationMetadata",
    )
    batch_predict_details: "BatchPredictOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="details",
        message="BatchPredictOperationMetadata",
    )
    export_data_details: "ExportDataOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="details",
        message="ExportDataOperationMetadata",
    )
    export_model_details: "ExportModelOperationMetadata" = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="details",
        message="ExportModelOperationMetadata",
    )
    export_evaluated_examples_details: "ExportEvaluatedExamplesOperationMetadata" = (
        proto.Field(
            proto.MESSAGE,
            number=26,
            oneof="details",
            message="ExportEvaluatedExamplesOperationMetadata",
        )
    )
    progress_percent: int = proto.Field(
        proto.INT32,
        number=13,
    )
    partial_failures: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=status_pb2.Status,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )


class DeleteOperationMetadata(proto.Message):
    r"""Details of operations that perform deletes of any entities."""


class DeployModelOperationMetadata(proto.Message):
    r"""Details of DeployModel operation."""


class UndeployModelOperationMetadata(proto.Message):
    r"""Details of UndeployModel operation."""


class CreateModelOperationMetadata(proto.Message):
    r"""Details of CreateModel operation."""


class ImportDataOperationMetadata(proto.Message):
    r"""Details of ImportData operation."""


class ExportDataOperationMetadata(proto.Message):
    r"""Details of ExportData operation.

    Attributes:
        output_info (google.cloud.automl_v1beta1.types.ExportDataOperationMetadata.ExportDataOutputInfo):
            Output only. Information further describing
            this export data's output.
    """

    class ExportDataOutputInfo(proto.Message):
        r"""Further describes this export data's output. Supplements
        [OutputConfig][google.cloud.automl.v1beta1.OutputConfig].

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            gcs_output_directory (str):
                The full path of the Google Cloud Storage
                directory created, into which the exported data
                is written.

                This field is a member of `oneof`_ ``output_location``.
            bigquery_output_dataset (str):
                The path of the BigQuery dataset created, in
                bq://projectId.bqDatasetId format, into which
                the exported data is written.

                This field is a member of `oneof`_ ``output_location``.
        """

        gcs_output_directory: str = proto.Field(
            proto.STRING,
            number=1,
            oneof="output_location",
        )
        bigquery_output_dataset: str = proto.Field(
            proto.STRING,
            number=2,
            oneof="output_location",
        )

    output_info: ExportDataOutputInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=ExportDataOutputInfo,
    )


class BatchPredictOperationMetadata(proto.Message):
    r"""Details of BatchPredict operation.

    Attributes:
        input_config (google.cloud.automl_v1beta1.types.BatchPredictInputConfig):
            Output only. The input config that was given
            upon starting this batch predict operation.
        output_info (google.cloud.automl_v1beta1.types.BatchPredictOperationMetadata.BatchPredictOutputInfo):
            Output only. Information further describing
            this batch predict's output.
    """

    class BatchPredictOutputInfo(proto.Message):
        r"""Further describes this batch predict's output. Supplements

        [BatchPredictOutputConfig][google.cloud.automl.v1beta1.BatchPredictOutputConfig].

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            gcs_output_directory (str):
                The full path of the Google Cloud Storage
                directory created, into which the prediction
                output is written.

                This field is a member of `oneof`_ ``output_location``.
            bigquery_output_dataset (str):
                The path of the BigQuery dataset created, in
                bq://projectId.bqDatasetId format, into which
                the prediction output is written.

                This field is a member of `oneof`_ ``output_location``.
        """

        gcs_output_directory: str = proto.Field(
            proto.STRING,
            number=1,
            oneof="output_location",
        )
        bigquery_output_dataset: str = proto.Field(
            proto.STRING,
            number=2,
            oneof="output_location",
        )

    input_config: io.BatchPredictInputConfig = proto.Field(
        proto.MESSAGE,
        number=1,
        message=io.BatchPredictInputConfig,
    )
    output_info: BatchPredictOutputInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=BatchPredictOutputInfo,
    )


class ExportModelOperationMetadata(proto.Message):
    r"""Details of ExportModel operation.

    Attributes:
        output_info (google.cloud.automl_v1beta1.types.ExportModelOperationMetadata.ExportModelOutputInfo):
            Output only. Information further describing
            the output of this model export.
    """

    class ExportModelOutputInfo(proto.Message):
        r"""Further describes the output of model export. Supplements

        [ModelExportOutputConfig][google.cloud.automl.v1beta1.ModelExportOutputConfig].

        Attributes:
            gcs_output_directory (str):
                The full path of the Google Cloud Storage
                directory created, into which the model will be
                exported.
        """

        gcs_output_directory: str = proto.Field(
            proto.STRING,
            number=1,
        )

    output_info: ExportModelOutputInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=ExportModelOutputInfo,
    )


class ExportEvaluatedExamplesOperationMetadata(proto.Message):
    r"""Details of EvaluatedExamples operation.

    Attributes:
        output_info (google.cloud.automl_v1beta1.types.ExportEvaluatedExamplesOperationMetadata.ExportEvaluatedExamplesOutputInfo):
            Output only. Information further describing
            the output of this evaluated examples export.
    """

    class ExportEvaluatedExamplesOutputInfo(proto.Message):
        r"""Further describes the output of the evaluated examples export.
        Supplements

        [ExportEvaluatedExamplesOutputConfig][google.cloud.automl.v1beta1.ExportEvaluatedExamplesOutputConfig].

        Attributes:
            bigquery_output_dataset (str):
                The path of the BigQuery dataset created, in
                bq://projectId.bqDatasetId format, into which
                the output of export evaluated examples is
                written.
        """

        bigquery_output_dataset: str = proto.Field(
            proto.STRING,
            number=2,
        )

    output_info: ExportEvaluatedExamplesOutputInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=ExportEvaluatedExamplesOutputInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
